/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "clientsocket.h"
#include "unit.h"
#include "devicetree.h"
#include "dsysinfo.h"

#include <QtWebSockets/QWebSocket>
#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonValue>
#include <QJsonArray>
#include <QHttpMultiPart>
#include <QDebug>
#include <QFile>
#include <QHttpMultiPart>
#include <QHttpPart>
#include <QFileInfo>
#include <QDir>
#include <QStandardPaths>
#include <QDBusReply>
#include <QDBusInterface>
#include <QUdpSocket>
#include <QPixmap>
#include <QEventLoop>

DCORE_USE_NAMESPACE

const int SERVER_TIMEOUT = 5000;
const int EM_NETOK = 70;
const QString STATUS_CODE("1");
DNetState *DNetState::m_Instance = Q_NULLPTR;

DNetState::DNetState()
{
    m_netInterface = new QDBusInterface("org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager", "org.freedesktop.NetworkManager", QDBusConnection::systemBus());
    connect(m_netInterface, SIGNAL(StateChanged(uint)), this, SLOT(noticeNetState(uint)));

    int netState = m_netInterface->property("State").toInt();

    if (netState == EM_NETOK) {
        m_netState = true;
    } else {
        m_netState = false;
    }
}

DNetState::~DNetState()
{
}

DNetState *DNetState::getInstance()
{
    if (m_Instance == Q_NULLPTR)
        m_Instance = new DNetState;

    return m_Instance;
}

void DNetState::noticeNetState(const uint &netState)
{
    if (netState == EM_NETOK) {
        m_netState = true;
    } else {
        m_netState = false;
    }

    Q_EMIT noticeNetWork(m_netState);
}

bool DNetState::getNetState()
{
    return m_netState;
}

DSocketClient::DSocketClient(QString serverUrl, QString serverPort, QObject *parent)
    : QObject(parent)
    , m_isConnectStatus(false)
    , m_testServer(false)
    , m_serverUrl(serverUrl)
    , m_serverPort(serverPort)
    , m_openApiToken("")
{
    if (m_serverUrl.isEmpty() && m_serverPort.isEmpty()) {
        initServerUrl(ServiceInfoConfig::getInstance()->getServiceUrl(),
                      ServiceInfoConfig::getInstance()->getServicePort());
    } else {
        m_testServer = true;
        initServerUrl(m_serverUrl, m_serverPort);
    }

    m_postTimer = new QTimer(this);
    m_postTimer->setSingleShot(true);
    connect(m_postTimer, &QTimer::timeout, this, &DSocketClient::postFaild);

    m_timer = new QTimer(this);
    m_netManager = new QNetworkAccessManager(this);

    if (m_testServer) {
        QSslConfiguration config;
        config.setPeerVerifyMode(QSslSocket::VerifyNone);
        m_httpRequest.setSslConfiguration(config);
    }

    connect(&m_webSocket, &QWebSocket::disconnected, this, &DSocketClient::disConnected);
    connect(&m_webSocket, &QWebSocket::textMessageReceived, this, &DSocketClient::textReceived);
    connect(&m_webSocket, &QWebSocket::connected, this, &DSocketClient::connected);
    connect(m_netManager, SIGNAL(finished(QNetworkReply *)), this, SLOT(requestFinished(QNetworkReply *)));
}

DSocketClient::~DSocketClient()
{
    m_webSocket.deleteLater();
}

//周期重连函数
void DSocketClient::renewConnect()
{
    m_webSocket.abort();
    m_webSocket.open(QUrl(m_webServerUrl));
}

//socket建立成功后，触发该函数
void DSocketClient::connected()
{
    m_isConnectStatus = true;
    Q_EMIT socketCreated();
    Q_EMIT connectStatus(ConnectedHost);
    m_timer->stop();
}

//收到Sev端的数据时，触发该函数
void DSocketClient::textReceived(const QString &msg)
{
    if (msg.isEmpty())
        return;

    QByteArray data = msg.toLocal8Bit();
    QJsonParseError jsonError;
    QJsonDocument jsonDocument(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject jsonObject = jsonDocument.object();
    QStringList jsonKeys = jsonObject.keys();

    //返回数据类型
    if (jsonKeys.contains("customer_token")) {
        //客服发送给客户消息
        Q_EMIT receiveMessage(jsonDocument.toJson().simplified());
    } else if (jsonKeys.contains("message_id")) {
        //客服返回的确认消息
        QString id = jsonObject["message_id"].toString();

        if (id != STATUS_CODE)
            Q_EMIT receiveMessageRet(jsonDocument.toJson().simplified());
    } else {
        qDebug() << "server back error";
    }
}

//socket连接断开后，触发该函数
void DSocketClient::disConnected()
{
    m_isConnectStatus = false;
    m_timer->start(3000);
}

void DSocketClient::closeSocket()
{
    m_webSocket.abort();
}

void DSocketClient::createSocket(const QString &custom)
{
    QString webUrl = m_webServerUrl + "?customer_token=" + custom;
    m_webSocket.open(QUrl(webUrl));
}

//发送消息，格式参考udesk消息定义
void DSocketClient::sendSession(const QByteArray &data)
{
    QString currentTime = QString("%1").arg(getWebTime());
    QString url = m_httpUrl + "/im/v1/session" + "?uname=" + m_openApiToken
                  + "&timestamp=" + currentTime + "&sign=" + getSign(currentTime, m_openApiToken);
    QNetworkRequest request;
    if (m_testServer) {
        QSslConfiguration config;
        config.setPeerVerifyMode(QSslSocket::VerifyNone);
        request.setSslConfiguration(config);
    }

    request.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("application/json"));
    request.setUrl(QUrl(url));
    m_netManager->post(request, data);
}

//数字签名根据时间，邮箱等组合加密而成
QString DSocketClient::getSign(const QString &timestr, const QString &userName)
{
    if (timestr.isEmpty())
        return QString("");

    QString str = userName + "&serviceudeskapitoken" + "&" + timestr;
    QByteArray sign = QCryptographicHash::hash(str.toLatin1(), QCryptographicHash::Sha256).toHex();
    return sign;
}

uint DSocketClient::getWebTime()
{
    QString url = "ntp6.aliyun.com";
    QTcpSocket *socket = new QTcpSocket();
    socket->connectToHost(url, 13);
    if (socket->waitForConnected(3000)) {
        if (socket->waitForReadyRead()) {
            QString str(socket->readAll());
            str = str.trimmed();
            str = str.section(" ", 1, 2);
            QString tempStr = QString::number((str.mid(9, 2).toInt() + 8));
            str.replace(9, 2, tempStr);
            str.insert(0, "20");
            QDateTime time = QDateTime::fromString(str, "yyyy-MM-dd hh:mm:ss");
            uint timeT = time.toTime_t();
            //qDebug()<<str<<timeT;
            socket->close();
            delete socket;
            return timeT;
        }
    }

    delete socket;
    qDebug() << "get web time fail..........................";
    return -1;
}

void DSocketClient::initServerUrl(const QString &serverUrl, const QString &serverPort)
{
    if (m_testServer) {
        m_webServerUrl += "ws://" + serverUrl + ":" + serverPort + "/im/v1/chat";
        m_httpUrl += "http://" + serverUrl + ":" + serverPort;
    } else {
        //获得在线客服服务器地址
        m_webServerUrl += "wss://" + serverUrl + ":" + serverPort + "/im/v1/chat";
        //获得文件发送的服务器地址
        m_httpUrl += "https://" + serverUrl + ":" + serverPort;
    }
}

//测试服初始化创建信息
void DSocketClient::createUser(QByteArray data)
{
    QString currentTime = QString("%1").arg(getWebTime());
    QString url = m_httpUrl + "/api/createuser" + "?uname=" + getUserName()
                  + "&timestamp=" + currentTime + "&sign=" + getSign(currentTime, getUserName());

    QNetworkRequest request;
    if (m_testServer) {
        QSslConfiguration config;
        config.setPeerVerifyMode(QSslSocket::VerifyNone);
        request.setSslConfiguration(config);
    }

    request.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("application/json"));
    request.setUrl(QUrl(url));
    m_netManager->post(request, data);
}

//websocket 接口- 发送用户满意度
void DSocketClient::sendSurveyOption(const QByteArray &data)
{
    m_webSocket.sendTextMessage(QString(data));
}

//websocket接口-发送消息，格式参考udesk消息定义
void DSocketClient::sendMessage(const QByteArray &data)
{
    if (m_webSocket.isValid()) {
        m_webSocket.sendTextMessage(QString(data));
    } else {
        qDebug() << "m_webSocket return false";
    }
}

void DSocketClient::uploadFile(const QString &filepath, const QString &msgid)
{
    if (filepath.isEmpty())
        return;

    QFileInfo info(filepath);
    QFile *file = new QFile(filepath, this);

    if (!file->open(QIODevice::ReadOnly)) {
        file->close();
        return;
    }

    QHttpMultiPart *multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType, this);
    //上传附件
    QHttpPart imagePart;
    imagePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"uploadfile\"; filename=\"" + info.fileName() + "\""));
    imagePart.setBodyDevice(file);
    QHttpPart textPart;
    textPart.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/plain"));
    textPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"message_id\""));
    textPart.setBody(msgid.toUtf8());
    multiPart->append(imagePart);
    multiPart->append(textPart);

    QString currentTime = QString("%1").arg(getWebTime());
    QString url = m_httpUrl + "/im/v1/uploadfile" + "?uname=" + m_openApiToken
                  + "&timestamp=" + currentTime + "&sign=" + getSign(currentTime, m_openApiToken);
    m_httpRequest.setUrl(QUrl(url));
    QNetworkReply *reply = m_netManager->post(m_httpRequest, multiPart);
    //更新上传进度
    connect(reply, &QNetworkReply::readyRead, [ = ]() {
        file->close();
    });
}

void DSocketClient::requestFinished(QNetworkReply *reply)
{
    qDebug() << reply->error();
    if (reply->error() == QNetworkReply::NoError) {
        //关闭定时器
        if (m_postTimer->isActive())
            m_postTimer->stop();

        QByteArray data = reply->readAll();

        QPixmap pix;
        if (pix.loadFromData(data)) {
            Q_EMIT avatarPixmap(pix);
            return;
        }

        //内容不为空，判断内容的返回
        if (!(data.isEmpty())) {
            QString subPath = reply->url().fileName();
            QJsonParseError jsonError;
            QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &jsonError));

            if (jsonError.error != QJsonParseError::NoError) {
                qDebug() << "json error!";
                Q_EMIT unlockConsultation();
                return;
            }

            QJsonObject jsonObject = jsonDoc.object();
            //返回内容成功
            int code = jsonObject["code"].toInt();

            if (subPath == "session") {
                if (code == 1000) {
                    Q_EMIT sessionStart(QJsonDocument(jsonObject).toJson());
                } else {
                    QString statusString = jsonObject["message"].toString();
                    Q_EMIT sessionFault(code, statusString);
                }
            }

            if (subPath == "createticker") {
                qint64 n_tickId;
                if (1000 == code) {
                    //工单创建
                    n_tickId = jsonObject["ticket_id"].toInt();
                } else {
                    n_tickId = 0;
                }
                Q_EMIT ticketComplete(n_tickId);
            }

            if (code == 1000) {
                if (subPath == "uploadfile") {
                    Q_EMIT uploadFileRet(m_httpUrl + "/im/v1/dl", data);
                } else if (subPath == "getDeviceTree") {
                    QJsonObject obj = jsonObject["value"].toObject();
                    Devicetree::getInstance()->praseJsonArray(QJsonDocument(obj).toJson().simplified());
                } else if (subPath == "agents") {
                    QJsonArray jsonArray = jsonObject["value"].toArray();
                    Q_EMIT customerInfo(QJsonDocument(jsonArray).toJson());
                    Devicetree::getInstance()->praseCustomerJsonArray(QJsonDocument(jsonArray).toJson());
                } else if (subPath == "agentinfo") {
                    QJsonObject obj = jsonObject["value"].toObject();
                    Q_EMIT initCustomerInfo(QJsonDocument(obj).toJson().simplified());
                } else if (subPath == "serviceUrl") {
                    //介绍和自助支持更多的网页链接
                    QString url = jsonObject["value"].toString();
                    Q_EMIT initSupportWebUrl(url);
                } else if (subPath == "createuser") {
                    QJsonObject object = jsonObject["data"].toObject();
                    m_openApiToken = object["openapitoken"].toString();
                    qDebug() << "createuser create success" << m_openApiToken;
                }else {
                    qDebug() << "server back error" << subPath;
                }
            } else {
                qDebug() << "server code error" << code;
            }
        } else {
            Q_EMIT unlockConsultation();
            qDebug() << "server not back";
        }
    } else {
        Q_EMIT unlockConsultation();
        qDebug() << "server back error";
    }

    reply->deleteLater();
}

QString DSocketClient::getUserName()
{
    QDBusInterface manual("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts/User1000", "com.deepin.daemon.Accounts.User", QDBusConnection::systemBus());
    QString name = manual.property("UserName").toString();
    return name;
}

QString DSocketClient::getSendUrlInfo()
{
    QString currentTime = QString("%1").arg(getWebTime());
    QString urlInfo = "uname=" + m_openApiToken + "&timestamp=" + currentTime + "&sign=" + getSign(currentTime, m_openApiToken);
    return urlInfo;
}

void DSocketClient::getUserAvatar(QString avatarUrl)
{
    m_httpRequest.setUrl(QUrl(avatarUrl));
    //设置接受请求重定向，否则无法拿到用户修改后的头像
    m_httpRequest.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
    m_netManager->get(m_httpRequest);
}

//发送工单信息,由留言咨询调用
void DSocketClient::sendTicket(const QString &username, const QByteArray &tickerValue, const QByteArray categoryValue, const QStringList &strFilePath)
{
    QString webTime = QString("%1").arg(getWebTime() - 1);
    //    qDebug() << " webTime time" << webTime;
    //    QString currentTime = QString::number(QDateTime::currentDateTime().toSecsSinceEpoch());
    //    qDebug() << " current time" << currentTime;
    //携带apitoken给服务器
    QString urlInfo = "uname=" + m_openApiToken + "&timestamp=" + webTime
                      + "&sign=" + getSign(webTime, m_openApiToken);
    QHttpMultiPart *multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType, this);
    //工单基础信息
    QHttpPart tickerPart;
    tickerPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"ticker\""));
    tickerPart.setBody(tickerValue);
    multiPart->append(tickerPart);

    //故障类别
    QHttpPart textCategory;
    textCategory.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"category\""));
    textCategory.setBody(categoryValue);
    multiPart->append(textCategory);

    //上传附件
    for (int index = 0; index < strFilePath.count(); index++) {
        QHttpPart imagePart;
        imagePart.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("image/jpeg"));
        QString strFileName = " ";
        int indexOne = strFilePath.at(index).lastIndexOf('/');
        strFileName = strFilePath.at(index);
        strFileName.replace(strFilePath.at(index).left(indexOne + 1), "");
        imagePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"uploadfiles\"; filename=\"" + strFileName + "\""));
        QFile *file = new QFile(strFilePath.at(index), this);

        if (!file->open(QIODevice::ReadOnly)) {
            qDebug() << "open file failed";
            file->deleteLater();
            continue;
        }

        imagePart.setBodyDevice(file);
        multiPart->append(imagePart);
    }

    QString postUrl = m_httpUrl + "/api/createticker?" + urlInfo;
    qDebug() << "createticker postUrl" << postUrl;
    m_httpRequest.setUrl(QString(postUrl));

    QNetworkReply *reply = m_netManager->post(m_httpRequest, multiPart);

    if (m_postTimer->isActive())
        m_postTimer->stop();
    m_postTimer->start(SERVER_TIMEOUT);
    QObject::connect(reply, &QNetworkReply::uploadProgress, this, &DSocketClient::updateUploadProcess);
}

void DSocketClient::updateUploadProcess(const int currentProgress, const int totalProgress)
{
    //emit updateProgress(QString::number((float)currentProgress / totalProgress));

    //有东西上传证明，不用做超时处理(提交失败)
    if ((currentProgress != 0) && (totalProgress != 0)) {
        if (m_postTimer->isActive())
            m_postTimer->stop();
    }
}

//获取页面地址
void DSocketClient::getSupportWebUrl(QString urlInfo)
{
    QString strPostUrl = m_httpUrl + "/api/serviceUrl?" + urlInfo;
    m_httpRequest.setUrl(QString(strPostUrl));
    m_netManager->get(m_httpRequest);
}

//联系我们 传递内容，为json串
void DSocketClient::searchCustomerInfo(const QByteArray &byteArray)
{
    QString strPostUrl = m_httpUrl + "/api/agentinfo?" + getSendUrlInfo();
    m_httpRequest.setUrl(QString(strPostUrl));
    m_netManager->post(m_httpRequest, byteArray);
}

//获取故障树列表
void DSocketClient::getDeviceTree(QString urlInfo)
{
    QString strPostUrl = m_httpUrl + "/api/getDeviceTree?" + urlInfo;
    m_httpRequest.setUrl(QString(strPostUrl));
    m_netManager->get(m_httpRequest);
}

void DSocketClient::getAllCustomerInfo(QString urlInfo)
{
    QString strPostUrl = m_httpUrl + "/api/agents?" + urlInfo;
    m_httpRequest.setUrl(QString(strPostUrl));
    m_netManager->get(m_httpRequest);
}
