/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "devicetree.h"

#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QStandardPaths>
#include <QDebug>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QEventLoop>
#include <QNetworkReply>
#include <QCoreApplication>

Devicetree *Devicetree::m_instance = Q_NULLPTR;
ServiceInfoConfig *ServiceInfoConfig::m_pInstance = Q_NULLPTR;

Devicetree *Devicetree::getInstance()
{
    if (m_instance == Q_NULLPTR)
        m_instance = new Devicetree;

    return m_instance;
}

Devicetree::Devicetree()
    : m_testServer(false)
    , m_serverUrl("")
    , m_serverPort("")
{
}

Devicetree::~Devicetree()
{
    if (m_instance != Q_NULLPTR) {
        delete m_instance;
        m_instance = Q_NULLPTR;
    }
}

QStringList Devicetree::getSystemNames()
{
    return m_system;
}

QStringList Devicetree::getBlockNames(const QString &system)
{
    return m_systemMap.value(system);
}

QStringList Devicetree::getItemNames(const QString &block)
{
    return m_blockMap.value(block);
}

ServiceCustomer Devicetree::getCustomService(const QString &index)
{
    return m_customers.value(index);
}

void Devicetree::loadCustomJsonFile()
{
    QString oldPath = QDir::currentPath();
    QString saveName = "customer.json";
    QString savePath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";
    QDir Dir;

    if (!Dir.exists(savePath))
        Dir.mkpath(savePath);

    //将程序的执行路径设置到filePath下
    Dir.setCurrent(savePath);

    //读json设备文件
    QFile readFile(saveName);

    if (!readFile.open(QIODevice::ReadOnly)) {
        qDebug() << tr("open file fail");
        return;
    }

    QByteArray allData = readFile.readAll();
    praseCustomerJsonArray(allData);
    readFile.close();
    QDir::setCurrent(oldPath);

    return;
}

//写json文件
void Devicetree::writeCustomerJsonFile()
{
    QJsonObject rootObj;

    QMapIterator<QString, ServiceCustomer> iter(m_customers);
    while (iter.hasNext()) {
        iter.next();

        ServiceCustomer customerInfo = iter.value();
        QJsonObject object;
        rootObj.insert("key", iter.key());

        object.insert("name", customerInfo.name);
        object.insert("team", customerInfo.team);
        object.insert("moblie", customerInfo.telephone);
        object.insert("email", customerInfo.email);
        object.insert("qrcode", customerInfo.QRCode);

        rootObj.insert("ServiceCustomer", object);

        QByteArray data = QJsonDocument(rootObj).toJson().simplified();
        data.push_back("\n");

        emit writeCustomJsonFiles(data);
    }
}

void Devicetree::setServerUrl(QString serverUrl, QString serverPort)
{
    m_serverUrl = serverUrl;
    m_serverPort = serverPort;
    m_testServer = true;
}

void Devicetree::praseQRcodeDownload(QByteArray data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QVariantList list = jsonDoc.toVariant().toList();
    for (int i = 0; i < list.count(); i++) {
        QVariantMap map = list[i].toMap();
        saveQRCode(map["qrcode"].toString());
    }
}

//解析文件中的每一行信息
void Devicetree::praseCustomerJsonArray(const QByteArray &data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QVariantList list = jsonDoc.toVariant().toList();
    ServiceCustomer customer;
    int id = 0;

    for (int i = 0; i < list.count(); i++) {
        QVariantMap map = list[i].toMap();

        id = map["id"].toInt();
        customer.name = map["name"].toString();
        customer.team = map["team"].toString();
        customer.telephone = map["moblie"].toString();
        customer.email = map["email"].toString();
        customer.QRCode = map["qrcode"].toString();
        QMap<QString, int>::iterator iter = m_customerIndex.begin();

        while (iter != m_customerIndex.end()) {
            if (id == iter.value()) {
                m_customers.insert(iter.key(), customer);
            }
            iter++;
        }
    }
    return;
}

void Devicetree::loadDeviceJsonFile()
{
    QString oldPath = QDir::currentPath();
    QString saveName = "uos-devicetree.json";
    QString savePath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";

    //将程序的执行路径设置到filePath下
    QDir::setCurrent(savePath);
    QFile readFile(saveName);

    if (!readFile.open(QIODevice::ReadOnly)) {
        qDebug() << tr("open file fail");
        return;
    }

    QByteArray allData = readFile.readAll();
    readFile.close();
    QDir::setCurrent(oldPath);
    praseJsonArray(allData);
    return;
}

QString Devicetree::saveQRCode(QString urlpath)
{
    QString serviceUrl;
    QString saveUrl;
    if (m_serverUrl.isEmpty() && m_serverPort.isEmpty()) {
        serviceUrl = "https://" + ServiceInfoConfig::getInstance()->getServiceUrl() + ":" + ServiceInfoConfig::getInstance()->getServicePort();
    } else {
        serviceUrl = "https://" + m_serverUrl + ":" + m_serverPort;
        qDebug() << "Now running is test server";
    }

    QFileInfo info(urlpath);
    QString saveName = info.fileName();
    QString saveDir = "/usr/share/" + QCoreApplication::applicationName() + "/ini/" + "QRCode";
    QString savePath = saveDir + "/" + saveName;

    urlpath = serviceUrl + "/" + urlpath;

    //构造请求
    QNetworkAccessManager manager;
    QNetworkRequest request;

    if (m_testServer) {
        QSslConfiguration config;
        config.setPeerVerifyMode(QSslSocket::VerifyNone);
        request.setSslConfiguration(config);
    }

    request.setUrl(urlpath);
    // 发送请求
    QNetworkReply *reply = manager.get(request);
    //开启一个局部的事件循环，等待响应结束，退出
    QEventLoop eventLoop;

    connect(reply, SIGNAL(finished()), &eventLoop, SLOT(quit()));
    eventLoop.exec();

    //判断是否出错,出错则结束
    if (reply->error() != QNetworkReply::NoError)
        return QString("download QRcode fail");

    emit saveFileQRCode(saveDir, saveName, reply->readAll());

    if (reply != Q_NULLPTR) {
        delete reply;
        reply = Q_NULLPTR;
    }

    return savePath;
}

void Devicetree::insertCustomerInfo(const QString &key, const ServiceCustomer &data)
{
    m_customers.insert(key, data);
}

void Devicetree::praseJsonArray(const QByteArray &data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject rootObject = jsonDoc.object();
    QJsonArray rootArray = rootObject["max_categories"].toArray();

    if (rootArray.isEmpty())
        return;

    m_system.clear();
    m_systemMap.clear();
    m_blockMap.clear();

    if (QLocale::system().name() == "zh_CN") {
        for (int i = 0; i < rootArray.size(); i++) {
            QJsonValue value = rootArray.at(i);

            if (value.isObject()) {
                QJsonObject systemObject = value.toObject();
                QString system = systemObject["name"].toString();
                QJsonArray blockArray = systemObject["mid_cateories"].toArray();

                m_system.push_back(system);

                QStringList blockList;

                for (int j = 0; j < blockArray.size(); j++) {
                    QJsonValue blockValue = blockArray.at(j);

                    if (blockValue.isObject()) {
                        QJsonObject blockObject = blockValue.toObject();
                        QString block = blockObject["name"].toString();
                        QJsonArray itemArray = blockObject["min_cateories"].toArray();

                        blockList.push_back(block);

                        QStringList itemList;

                        for (int k = 0; k < itemArray.size(); k++) {
                            QJsonValue itemValue = itemArray.at(k);

                            if (itemValue.isObject()) {
                                QJsonObject itemObject = itemValue.toObject();
                                QString itemName = itemObject["name"].toString();
                                int customerId = itemObject["agent_id"].toInt();

                                m_customerIndex.insert(system + "-" + block + "-" + itemName, customerId);
                                itemList.push_back(itemName);
                            }
                        }
                        m_blockMap.insert(block, itemList);
                    }
                }
                m_systemMap.insert(system, blockList);
            }
        }
    } else {
        for (int i = 0; i < rootArray.size(); i++) {
            QJsonValue value = rootArray.at(i);

            if (value.isObject()) {
                QJsonObject systemObject = value.toObject();
                QString system = systemObject["ename"].toString();
                QJsonArray blockArray = systemObject["mid_cateories"].toArray();

                m_system.push_back(system);

                QStringList blockList;

                for (int j = 0; j < blockArray.size(); j++) {
                    QJsonValue blockValue = blockArray.at(j);

                    if (blockValue.isObject()) {
                        QJsonObject blockObject = blockValue.toObject();
                        QString block = blockObject["ename"].toString();
                        QJsonArray itemArray = blockObject["min_cateories"].toArray();
                        blockList.push_back(block);

                        QStringList itemList;
                        for (int k = 0; k < itemArray.size(); k++) {
                            QJsonValue itemValue = itemArray.at(k);

                            if (itemValue.isObject()) {
                                QJsonObject itemObject = itemValue.toObject();
                                QString item = itemObject["ename"].toString();
                                int customerId = itemObject["agent_id"].toInt();

                                m_customerIndex.insert(system + "-" + block + "-" + item, customerId);
                                itemList.push_back(item);
                            }
                        }
                        m_blockMap.insert(block, itemList);
                    }
                }
                m_systemMap.insert(system, blockList);
            }
        }
    }

    //更新完设备数变量，写设备文件
    emit updataDeviceTreeCombox();
    emit writeDeviceJsonFile(data);
}

ServiceInfoConfig::ServiceInfoConfig()
{
}

ServiceInfoConfig::~ServiceInfoConfig()
{
}

ServiceInfoConfig *ServiceInfoConfig::getInstance()
{
    if (m_pInstance == Q_NULLPTR)
        m_pInstance = new ServiceInfoConfig;

    return m_pInstance;
}

void ServiceInfoConfig::loadConfigJsonFile()
{
    QString oldPath = QDir::currentPath();
    QString saveName = "service_support_config.json";
    QString savePath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";
    QDir iniDir;

    if (!iniDir.exists(savePath))
        iniDir.mkpath(savePath);

    iniDir.setCurrent(savePath);
    QFile loadFile(saveName);
    if (!loadFile.open(QIODevice::ReadOnly)) {
        qDebug() << "open service config json file fail";
        return;
    }

    QByteArray allData = loadFile.readAll();
    loadFile.close();
    praseJsonArray(allData);
    iniDir.setCurrent(oldPath);
}

void ServiceInfoConfig::praseJsonArray(const QByteArray &data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject rootObj = jsonDoc.object();
    m_serviceUrl = rootObj["service_url"].toString();
    m_servicePort = rootObj["service_port"].toString();
}

void ServiceInfoConfig::writeJsonFile(const QByteArray &data)
{
    QString saveName = "service_support_config.json";
    QString savePath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";
    QDir iniDir;

    if (!iniDir.exists(savePath))
        iniDir.mkpath(savePath);

    iniDir.setCurrent(savePath);
    QFile file(saveName);

    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate)) {
        qDebug() << "File open error";
        return;
    } else {
        qDebug() << "File open!";
    }

    file.write(data);
    file.close();
}

QString ServiceInfoConfig::getServiceUrl()
{
    return m_serviceUrl;
}

QString ServiceInfoConfig::getServicePort()
{
    return m_servicePort;
}
