#include "systeminfo.h"
#include "diskid.h"
#include "unit.h"
#include "dsysinfo.h"

#include <QSysInfo>
#include <QThread>
#include <QProcess>
#include <QDBusInterface>
#include <QDBusReply>
#include <QProcess>
#include <QFile>
#include <QNetworkInterface>
#include <QSettings>
#include <QDebug>

#include <openssl/md5.h>

DCORE_USE_NAMESPACE

namespace SystemInfo {

typedef QMap<QString, QString> DiskInfo;

QString stripValue(QString value)
{
    if (value.length() >= 2) {
        if (value.at(value.length() - 1) == '"') {
            value = value.left(value.length() - 1);
        }

        if (value.at(0) == '"') {
            value = value.right(value.length() - 1);
        }
    }
    return value;
}

QList<DiskInfo> parseLsblk(const QString &lsblkOut)
{
    QList<DiskInfo> disks;
    auto entrys = lsblkOut.split("\n");
    for (auto entry : entrys) {
        DiskInfo info;
        auto propertys = entry.split(" ");
        for (auto pro : propertys) {
            auto kv = pro.split("=");
            if (kv.length() != 2) {
                continue;
            }
            info.insert(kv.value(0), stripValue(kv.value(1)));
        }
        disks << info;
    }
    return disks;
}

QList<DiskInfo> getMountDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-P");
    lsblk.start();
    lsblk.waitForFinished();

    auto rootMountpoint = QStringList() << "/"
                                        << "/boot";
    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    auto disks = parseLsblk(lsblkOut);
    QList<DiskInfo> mountDisk;
    for (auto disk : disks) {
        auto mp = disk.value("MOUNTPOINT");

        if (!rootMountpoint.contains(mp)) {
            continue;
        }
        mountDisk << disk;
    }
    return mountDisk;
}

QList<DiskInfo> getPhyDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-P"
                                     << "-d");
    lsblk.start();
    lsblk.waitForFinished();

    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    return parseLsblk(lsblkOut);
}

QString getDiskMaj(const DiskInfo &disk)
{
    auto mm = disk.value("MAJ:MIN");
    auto mms = mm.split(":");
    if (mms.length() != 2) {
        return "";
    }
    return mms.first();
}

QString getRootDisk()
{
    auto mountDisk = getMountDisk();
    auto phyDisk = getPhyDisk();

    for (auto disk : mountDisk) {
        auto maj = getDiskMaj(disk);
        for (auto phy : phyDisk) {
            auto phyMaj = getDiskMaj(phy);
            if (phyMaj == maj) {
                return QString("/dev/%1").arg(phy.value("NAME"));
            }
        }
    }
    return "";
}

void fillMachineIDasMchineCode(unsigned char *id)
{
    QFile machineIDFile("/etc/machine-id");
    machineIDFile.open(QIODevice::ReadOnly);
    auto data = machineIDFile.readAll();
    char buffer[1024] = {0};
    strcpy(buffer, data.data());
    //make md5 of buffer
    MD5_CTX c;
    unsigned char out[MD5_DIGEST_LENGTH];
    MD5_Init(&c);
    MD5_Update(&c, buffer, 1024);
    MD5_Final(out, &c);
    memcpy(id, out, MD5_DIGEST_LENGTH);
}

QString encodeDiskID(unsigned char *id)
{
    for (int i = 0; i < MD5_DIGEST_LENGTH / 2; i++) {
        id[i] = id[i] / 2 + id[MD5_DIGEST_LENGTH - 1 - i] / 2;
    }

    QString machine_code;
    for (int i = 0; i < 4; i++) {
        machine_code.append(QString().sprintf("%02x", id[i * 4]));
        machine_code.append(QString().sprintf("%02x", id[i * 4 + 1]));
        if (i < 3) {
            machine_code += "-";
        }
    }
    return machine_code;
}

QString getSystemMachineCode()
{
    unsigned char id[MD5_DIGEST_LENGTH];
    auto diskName = getRootDisk();

    auto ret = diskid(diskName.toStdString().c_str(), id);

#ifdef DEEPIN_AUTO_FAKE_MACHINECODE
    // get disk id failed
    if (1 == ret) {
        qDebug() << "try to read machine code";
        fillMachineIDasMchineCode(id);
    }
#else
    Q_UNUSED(ret)
#endif
    return encodeDiskID(id);
}

QString getSystemVersion()
{
    QString version;

    if (DSysInfo::isDeepin()) {
        version = QString("%1 %2").arg(DSysInfo::deepinVersion()).arg(DSysInfo::deepinTypeDisplayName());
    } else {
        version = QString("%1 %2").arg(DSysInfo::productTypeString()).arg(DSysInfo::productVersion());
    }
    return version;
}

quint32 getActiveState()
{
    QDBusInterface licenseInfo("com.deepin.license.activator", "/com/deepin/license/activator", "com.deepin.license.activator", QDBusConnection::sessionBus());
    if (!licenseInfo.isValid()) {
        qWarning() << "com.deepin.license error ," << licenseInfo.lastError().name();
    }

    QDBusReply<quint32> reply = licenseInfo.call(QDBus::AutoDetect, "GetIndicatorData");
    return reply;
}

QString getActiveCode()
{
    QString activeCode = "1111-1111-1111-1111";
    return activeCode;
}

QString getSystemCopyright()
{
    QSettings uos_version("/etc/deepin-version", QSettings::IniFormat);
    uos_version.setIniCodec("UTF8");
    QString systemType = uos_version.value("Release/Copyright").toString();
    return systemType;
}

QString getSystemEdition()
{
    QSettings uos_version("/etc/deepin-version", QSettings::IniFormat);
    uos_version.setIniCodec("UTF8");
    QString systemType = uos_version.value("Release/Edition").toString();
    return systemType;
}

QString getHostMacAddress()
{
    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces(); // 获取所有网络接口列表
    int nCnt = nets.count();
    QString strMacAddr = "";
    for (int i = 0; i < nCnt; i++) {
        // 如果此网络接口被激活并且正在运行并且不是回环地址，则就是我们需要找的Mac地址
        if (nets[i].flags().testFlag(QNetworkInterface::IsUp) && nets[i].flags().testFlag(QNetworkInterface::IsRunning) && !nets[i].flags().testFlag(QNetworkInterface::IsLoopBack)) {
            strMacAddr = nets[i].hardwareAddress();
            break;
        }
    }
    return strMacAddr;
}

QString getSystemType()
{
    QSettings uos_version("/etc/deepin-version", QSettings::IniFormat);
    uos_version.setIniCodec("UTF8");
    QString systemType = uos_version.value("Release/Type").toString();
    return systemType;
}

QString getType()
{
    return QString("%1").arg(QSysInfo::WordSize);
}

QString getProcessor()
{
    return QString("%1 x %2").arg(DSysInfo::cpuModelName()).arg(QThread::idealThreadCount());
}

QString getKernel()
{
    QProcess process;
    process.start("uname -r");
    process.waitForFinished();
    QByteArray output = process.readAllStandardOutput();
    int idx = output.indexOf('\n');
    if (-1 != idx) {
        output.remove(idx, 1);
    }
    return output;
}

} // namespace SystemInfo
