/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "customerchatwidget.h"
#include "dsysinfo.h"
#include "common/clientsocket.h"
#include "common/licensestatus.h"

#include <DIconButton>
#include <DPalette>
#include <DDialog>
#include <DFileDialog>
#include <DMenu>
#include <DPushButton>
#include <DMessageManager>
#include <DCommandLinkButton>

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QKeyEvent>
#include <QFileDialog>
#include <QScrollBar>
#include <QUuid>
#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QEventLoop>
#include <QStandardPaths>
#include <QFileIconProvider>
#include <QApplication>
#include <QDesktopWidget>
#include <QDesktopServices>
#include <QFileInfo>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QTimer>
#include <QMimeData>
#include <QTextBlock>
#include <QDebug>

const int SUCCESSCODE = 1000;
const int BUSYCODE = 2001;
const int NOONLINECODE = 2002;
const QByteArray secretKey = "uos-server-support";
const qint64 FILESIZE = 10 * 1024 * 1024;

bool sortDate(QString dateone, QString datetwo)
{
    if (QDate::fromString(dateone, "yyyy.MM.dd") > QDate::fromString(datetwo, "yyyy.MM.dd"))
        return true;
    return false;
}

TitleFrame::TitleFrame(QWidget *parent)
    : DFrame(parent)
    , m_customerIcon(":/images/Avatar.svg")
{
    QHBoxLayout *titleLayout = new QHBoxLayout;
    m_customerAvatar = new DIconButton(this);
    this->setAttribute(Qt::WA_TransparentForMouseEvents, true);
    m_customerAvatar->setIcon(QIcon::fromTheme(m_customerIcon));
    m_customerAvatar->setIconSize(QSize(32, 32));
    m_customerAvatar->setFlat(true);
    m_customerAvatar->setFocusPolicy(Qt::FocusPolicy::NoFocus);

    QFont fontType("SourceHanSansSC-Medium");
    fontType.setBold(true);
    fontType.setWeight(65);
    fontType.setPointSize(9);

    m_customerAppellation = new DLabel;
    m_customerAppellation->setFont(fontType);
    m_customerAppellation->setText(tr("OnlineSupport"));
    titleLayout->setContentsMargins(10, 10, 10, 3);
    titleLayout->addWidget(m_customerAvatar);
    titleLayout->addWidget(m_customerAppellation);
    titleLayout->addStretch();
    setLayout(titleLayout);
    setFixedHeight(49);
}

TitleFrame::~TitleFrame()
{
}

void TitleFrame::setTitle(const QString &customerName, const QString &customerIcon)
{
    if (!customerIcon.isEmpty()) {
        m_customerAvatar->setIcon(QIcon::fromTheme(m_customerIcon));
        m_customerAvatar->setIconSize(QSize(32, 32));
    }

    m_customerAppellation->setText(customerName);
}

MessageFrame::MessageFrame(Message message, QWidget *parent)
    : DFrame(parent)
    , m_messageLabel(Q_NULLPTR)
    , m_messageFrame(Q_NULLPTR)
    , m_sendMessageSuccess(false)
    , m_messageEdit(Q_NULLPTR)
{
    this->setAutoFillBackground(true);
    m_messageFrameLayout = new QHBoxLayout;
    m_messageContent = message;

    m_menu = new DMenu;
    QAction *openAction = new QAction(tr("open file"));
    QAction *saveAsAction = new QAction(tr("save as"));
    m_menu->addAction(openAction);
    m_menu->addAction(saveAsAction);
    connect(openAction, &QAction::triggered, this, &MessageFrame::openFile);
    connect(saveAsAction, &QAction::triggered, this, &MessageFrame::saveAsFile);

    initUI();

    setLayout(m_messageFrameLayout);
    m_messageFrameLayout->setContentsMargins(0, 0, 0, 0);
    setFrameShape(QFrame::NoFrame);
    this->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
}

MessageFrame::~MessageFrame()
{
}

QString MessageFrame::getMessageId()
{
    return m_messageContent.messageId;
}

void MessageFrame::setMessageRecive()
{
    m_sendMessageSuccess = true;

    if (m_messageContent.sendMessageDirection == "Receive")
        return;

    m_messageSuccessLabel->setIcon(QIcon::fromTheme(":/images/read.svg"));

    QTimer::singleShot(3000, this, [ = ]() {
        m_messageSuccessLabel->setVisible(false);
    });
}

Message MessageFrame::getMessage()
{
    return m_messageContent;
}

DFrame *MessageFrame::createFileFrame()
{
    QFont fontType;
    fontType = QFont("SourceHanSansSC-Normal,SourceHanSansSC", 9);

    QFileInfo fileInfo(m_messageContent.messageContent);
    QFileIconProvider iconProvider;
    QIcon icon = iconProvider.icon(fileInfo);

    DLabel *fileNameLabel = new DLabel;
    fileNameLabel->setFont(fontType);

    DLabel *fileSizeLabel = new DLabel;
    fileSizeLabel->setFont(fontType);

    DLabel *fileIconLabel = new DLabel;
    fileIconLabel->setFixedSize(50, 50);
    fileIconLabel->setPixmap(icon.pixmap(50, 50));
    fileIconLabel->setScaledContents(true);

    int bubbleWidth = 0;
    int bubbleHeight = 0;

    QFontMetrics fontMetrics(fontType);
    bubbleWidth = fontMetrics.width(m_messageContent.fileName);
    bubbleHeight = fontMetrics.height();

    QString sendFileName = m_messageContent.fileName;
    int fileNameNum = sendFileName.count();

    QString foreLength;
    QString behindLength;

    if (bubbleWidth > 130) {
        for (int i = 0; i < fileNameNum; i++) {
            QString newFileName = sendFileName.mid(i, 1);
            foreLength = foreLength + newFileName;
            if (fontMetrics.width(foreLength) > 130)
                break;
        }
        for (fileNameNum; fileNameNum > 0; fileNameNum--) {
            QString newFileName = sendFileName.mid(fileNameNum, 1);
            behindLength = newFileName + behindLength;
            if (fontMetrics.width(behindLength) > 120)
                break;
        }

        sendFileName = foreLength + "\n" + "..." + behindLength;
    }

    fileNameLabel->setText(sendFileName);
    fileNameLabel->setToolTip(m_messageContent.fileName);

    int filesize = m_messageContent.fileSize.toInt();
    QString size;
    QString unit = "B";

    if (filesize > 1024) {
        filesize /= 1024;
        unit = "Kb";
        if (filesize > 1024) {
            filesize /= 1024;
            unit = "Mb";
            if (filesize > 1024) {
                filesize /= 1024;
                unit = "Gb";
            }
        }
    }

    size.append(QString::number(filesize)).append(unit);
    fileSizeLabel->setText(size);

    QVBoxLayout *verticalLayout = new QVBoxLayout;
    verticalLayout->addWidget(fileNameLabel);
    verticalLayout->addWidget(fileSizeLabel);
    verticalLayout->setSpacing(3);

    QHBoxLayout *horizontalLayot = new QHBoxLayout;
    horizontalLayot->addLayout(verticalLayout);
    horizontalLayot->addWidget(fileIconLabel);
    horizontalLayot->setSpacing(20);
    horizontalLayot->setContentsMargins(11, 11, 11, 11);

    DFrame *frameWidget = new DFrame;
    frameWidget->setLayout(horizontalLayot);
    frameWidget->setBackgroundRole(DPalette::ItemBackground);
    frameWidget->setFixedSize(250, 88);
    frameWidget->setContentsMargins(3, 3, 3, 3);
    frameWidget->installEventFilter(this);
    frameWidget->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(frameWidget, &QLabel::customContextMenuRequested, [ = ](const QPoint) {
        m_menu->exec(QCursor::pos());
    });

    return frameWidget;
}

DLabel *MessageFrame::createImageLabel()
{
    DLabel *imageLabel = new DLabel(this);
    QPixmap pixmap(m_messageContent.messageContent);
    int width = 100;
    int height = 100;
    QDesktopWidget *desktop = QApplication::desktop();
    int screenwidth = desktop->availableGeometry().width();
    int screenheigth = desktop->availableGeometry().width();
    int pixwidth = pixmap.width();
    int pixheight = pixmap.height();

    if (pixwidth >= screenwidth || pixheight >= screenheigth) {
        width = 360;
        height = width * (pixheight / (pixwidth * 1.0));
    } else {
        float iconWidth = (pixwidth / (screenwidth * 1.0));
        float iconHeight = (pixheight / (screenheigth * 1.0));
        width = 500 * iconWidth;
        height = 500 * iconHeight;
    }

    imageLabel->setPixmap(pixmap.scaled(width, height));
    imageLabel->installEventFilter(this);
    imageLabel->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(imageLabel, &QLabel::customContextMenuRequested, [ = ](const QPoint) {
        m_menu->exec(QCursor::pos());
    });

    return imageLabel;
}

void MessageFrame::saveAsFile()
{
    QFileInfo fileInfo(m_messageContent.messageContent);
    QString filename = DFileDialog::getSaveFileName(this, tr("Save File"), "/home", "file (*." + fileInfo.completeSuffix() + ")");

    if (filename.isEmpty())
        return;

    QFile::copy(m_messageContent.messageContent, filename);
}

void MessageFrame::openFile()
{
    QDesktopServices::openUrl(QUrl::fromLocalFile(m_messageContent.messageContent));
}

void MessageFrame::initUI()
{
    QFont fontType;
    fontType.setPointSize(10);

    if (m_messageContent.sendMessageDirection == "Receive") {
        QVBoxLayout *overallLayout = new QVBoxLayout;

        m_nameLabel = new DLabel(m_messageContent.sendMessageName);
        m_nameLabel->setFont(fontType);

        m_iconLabel = new DIconButton(this);
        m_iconLabel->setIcon(QIcon::fromTheme(m_messageContent.sendMessageIcon));
        m_iconLabel->setIconSize(QSize(32,32));
        m_iconLabel->setFlat(true);
        m_iconLabel->setAttribute(Qt::WA_TransparentForMouseEvents, true);

        m_timeLabel = new DLabel;
        m_timeLabel->setFont(fontType);
        QDateTime datetime = QDateTime::fromString(m_messageContent.sendMessageTime, "yyyy.MM.dd hh:mm:ss");
        QString time = datetime.toLocalTime().toString("H:mm AP");
        m_timeLabel->setText(time);

        QHBoxLayout *receiveLayout = new QHBoxLayout;
        receiveLayout->setSpacing(10);
        receiveLayout->addStrut(10);
        receiveLayout->addWidget(m_iconLabel);
        receiveLayout->addWidget(m_nameLabel);
        receiveLayout->addWidget(m_timeLabel);
        receiveLayout->addStretch(5);
        receiveLayout->setSpacing(10);
        overallLayout->addLayout(receiveLayout);

        if (m_messageContent.sendMessageType == "Message") {
            m_messageEdit = new QTextEdit(this);
            m_messageEdit->setFont(fontType);
            m_messageEdit->setWordWrapMode(QTextOption::NoWrap);
            QTextDocument *document = m_messageEdit->document();
            document->setDocumentMargin(2);

            QStringList messageContentList = parseEmtion(m_messageContent.messageContent);
            QStringList messageList = adjustRect(messageContentList);

            inputContent(messageList);
            m_messageEdit->setTextColor(QColor(0, 0, 0));
            m_messageEdit->setReadOnly(true);
            m_messageEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
            m_messageEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
            m_messageEdit->setStyleSheet("background-color:rgba(0,0,0,0.03); border-radius:5px;");
            m_messageEdit->setMouseTracking(true);
            m_messageEdit->viewport()->setMouseTracking(true);
            m_messageEdit->viewport()->installEventFilter(this);
            m_messageEdit->setAlignment(Qt::AlignLeft);

            QTextCursor textCursor = m_messageEdit->textCursor();
            for (QTextBlock it = document->begin(); it != document->end(); it = it.next()) {
                QTextBlockFormat textBlockFormat = it.blockFormat();

                textBlockFormat.setLineHeight(25, QTextBlockFormat::FixedHeight);//设置固定行高
                textBlockFormat.setLeftMargin(7);
                textCursor.setPosition(it.position());
                textCursor.setBlockFormat(textBlockFormat);
                m_messageEdit->setTextCursor(textCursor);
            }

            QEvent event(QEvent::PaletteChange);
            QCoreApplication::sendEvent(this, &event);

            QHBoxLayout *messageLayout = new QHBoxLayout;
            messageLayout->addWidget(m_messageEdit);
            messageLayout->addStretch(10);
            overallLayout->addLayout(messageLayout);

        } else if (m_messageContent.sendMessageType == "Image") {
            m_messageLabel = createImageLabel();
            overallLayout->addWidget(m_messageLabel);
        } else if (m_messageContent.sendMessageType == "File") {
            m_messageFrame = createFileFrame();
            overallLayout->addWidget(m_messageFrame);
        } else {
        }

        overallLayout->setSpacing(10);
        m_messageFrameLayout->addLayout(overallLayout);
        m_messageFrameLayout->addStretch(5);

    } else if (m_messageContent.sendMessageDirection == "Send") {
        QVBoxLayout *sendLayout = new QVBoxLayout;
        QHBoxLayout *sendMessageLayout = new QHBoxLayout;
        m_iconLabel = new DIconButton(this);
        m_timeLabel = new DLabel;
        m_nameLabel = new DLabel(m_messageContent.sendMessageName);
        m_timeLabel->setFont(fontType);
        m_nameLabel->setFont(fontType);
        QDateTime datetime = QDateTime::fromString(m_messageContent.sendMessageTime, "yyyy.MM.dd hh:mm:ss");
        QString time = datetime.toLocalTime().toString("H:mm AP");
        m_timeLabel->setText(time);
        sendMessageLayout->addStretch(10);
        sendMessageLayout->addWidget(m_timeLabel);
        sendMessageLayout->addWidget(m_nameLabel);
        sendMessageLayout->addWidget(m_iconLabel);
        sendMessageLayout->setSpacing(10);

        m_iconLabel->setIcon(QIcon::fromTheme(m_messageContent.sendMessageIcon));
        m_iconLabel->setIconSize(QSize(32, 32));
        m_iconLabel->setFlat(true);
        m_iconLabel->setAttribute(Qt::WA_TransparentForMouseEvents, true);


        sendLayout->addLayout(sendMessageLayout);

        if (m_messageContent.sendMessageType == "Message") {
            m_messageEdit = new QTextEdit(this);
            m_messageEdit->setFont(fontType);
            m_messageEdit->setWordWrapMode(QTextOption::NoWrap);

            QTextDocument *document = m_messageEdit->document();
            document->setDocumentMargin(2);

            QStringList messageContentList = parseEmtion(m_messageContent.messageContent);
            QStringList messageList = adjustRect(messageContentList);

            inputContent(messageList);

            m_messageEdit->setTextColor(Qt::white);
            m_messageEdit->setReadOnly(true);
            m_messageEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
            m_messageEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
            m_messageEdit->setStyleSheet("background-color:rgba(0,129,255,1); color:white;border-radius:5px;");
            m_messageEdit->setMouseTracking(true);
            m_messageEdit->viewport()->setMouseTracking(true);
            m_messageEdit->viewport()->installEventFilter(this);
            m_messageEdit->setAlignment(Qt::AlignLeft);

            QTextCursor textCursor = m_messageEdit->textCursor();
            for (QTextBlock it = document->begin(); it != document->end(); it = it.next()) {
                QTextBlockFormat textBlockFormat = it.blockFormat();

                textBlockFormat.setLineHeight(25, QTextBlockFormat::FixedHeight);//设置固定行高
                textBlockFormat.setLeftMargin(7);
                textCursor.setPosition(it.position());
                textCursor.setBlockFormat(textBlockFormat);
                m_messageEdit->setTextCursor(textCursor);
            }

            QHBoxLayout *layout = new QHBoxLayout;
            layout->addStretch(10);
            layout->addWidget(m_messageEdit);
            sendLayout->addLayout(layout);
        } else if (m_messageContent.sendMessageType == "Image") {
            m_messageLabel = createImageLabel();
            sendLayout->addWidget(m_messageLabel);
        } else if (m_messageContent.sendMessageType == "File") {
            //文件对话框
            m_messageFrame = createFileFrame();
            sendLayout->addWidget(m_messageFrame);
        } else {
        }
        sendLayout->setSpacing(10);
        m_messageFrameLayout->addStretch(5);
        //消息发送成功失败标志
        m_messageSuccessLabel = new DIconButton(this);
        m_messageSuccessLabel->setFlat(true);
        m_messageSuccessLabel->setIcon(QIcon::fromTheme("://images/unread.svg"));

        QVBoxLayout *m_vlayout = new QVBoxLayout();
        m_vlayout->addStretch();
        m_vlayout->addWidget(m_messageSuccessLabel);

        m_messageFrameLayout->addLayout(m_vlayout);
        m_messageFrameLayout->addLayout(sendLayout);
    } else {
    }
}

void MessageFrame::adjustContent()
{
    QTextDocument *document = qobject_cast<QTextDocument *>(sender());
    document->adjustSize();

    if (document) {
        document->setDocumentMargin(8);
        QTextEdit *editor = qobject_cast<QTextEdit *>(document->parent()->parent());
        if (editor) {
            QString messageText = editor->toPlainText().trimmed();
            if (messageText.isEmpty())
                return;

            QFontMetrics fontMetrics = editor->viewport()->fontMetrics();
            int messageCount = messageText.count("\n");
            int messageWidth = fontMetrics.width(messageText);
            int messageHeight = fontMetrics.height();

            if (messageWidth > editor->maximumWidth()) {
                float floatHeight = (messageWidth / (300 * 1.0)) + messageCount;
                int intHeight = (messageWidth / (300 * 1.0)) + messageCount;
                float difference = floatHeight - intHeight;
                if (floatHeight - intHeight > 0) {
                    intHeight++;
                }
                if (difference < 0.5) {
                    editor->setFixedSize(300, intHeight * messageHeight + 28);
                } else {
                    editor->setFixedSize(300, intHeight * messageHeight + 30);
                }
            } else {
                editor->setFixedSize(messageWidth + 28, 36);
            }
        }
    }
}

QStringList MessageFrame::adjustRect(QStringList messageList)
{
    QFont fontType;
    fontType.setPointSize(10);
    QFontMetrics fontMetrics(fontType);
    QStringList results;

    int flag = 1;
    double charWidth = 0;
    double lineWidth = 0;
    double lineHeight = 0;
    double tempWidth = 0;
    double tempHeight = fontMetrics.height();
    qDebug() << "fontMetrics.height() =========" << fontMetrics.height();
    QString tempStr;

    for (int i = 0; i < messageList.length(); i++) {
        QString imgName = messageList[i].mid(messageList[i].lastIndexOf('/') + 1);
        if (!nameToEscape.value(imgName).isEmpty()) {
            if ((tempWidth + 25) > 300) {
                if (tempWidth > lineWidth)
                    lineWidth = tempWidth;

                lineHeight += tempHeight;
                tempWidth = 0;
                tempHeight = 0;
                messageList.insert(i, "\n");
                results << "\n";
                flag++;
                i++;
            }

            tempWidth += 25.0;
            tempHeight = 25.0;
            results << messageList[i];
            continue;
        }

        if (messageList[i] == "\n") {
            if (tempWidth > lineWidth)
                lineWidth = tempWidth;
            lineHeight += tempHeight;
            flag++;
            tempWidth = 0;
            results << "\n";
            continue;
        }

        for (int j = 0; j < messageList[i].length(); j++) {
            if (messageList[i][j] == '\t') {
                tempStr += "        ";
                charWidth = fontMetrics.width(tempStr);
            } else {
                tempStr += messageList[i][j];
                charWidth = fontMetrics.width(tempStr);
            }

            if ((tempWidth + charWidth) > 300) {
                messageList[i].insert(j, "\n");
                tempStr.chop(1);
                while (' ' == tempStr.at(tempStr.length() - 1))
                    tempStr.chop(1);

                tempWidth += fontMetrics.width(tempStr);
                if (tempWidth > lineWidth)
                    lineWidth = tempWidth;

                lineHeight += tempHeight;
                flag++;
                tempHeight = fontMetrics.height() + 4;
                tempWidth = 0;
                charWidth = 0;
                results << tempStr << "\n";
                tempStr.clear();
            }
        }

        results << tempStr;
        tempStr.clear();
        tempWidth += charWidth;
        if (tempHeight != 25.0)
            tempHeight = fontMetrics.height() + 4;

    }

    if (tempWidth > lineWidth)
        lineWidth = tempWidth;

    lineWidth += 20;
    if (lineWidth < 35)
        lineWidth = 35;

    lineHeight = flag * 25 + 12;
    m_messageEdit->setFixedSize(lineWidth, lineHeight);

    return results;
}

void MessageFrame::adjustText()
{
    QTextDocument *document = qobject_cast<QTextDocument *>(sender());
    document->adjustSize();
    if (document) {
        QTextEdit *editor = qobject_cast<QTextEdit *>(document->parent()->parent());
        if (editor) {
            int newheight = document->size().rheight();
            if (newheight != editor->height()) {
                editor->setFixedHeight(newheight);
            }
        }
    }
}

void MessageFrame::inputContent(QStringList messageList)
{
    QImage image;
    for (auto it : messageList) {
        QString imgPath = it.mid(0, 17);
        if (image.load(it) && (imgPath == ":/images/emotion/")) {
            m_messageEdit->insertHtml("<span style=\" font-size:10pt;\"><img src=\"" + it + "\"></span>");
        } else if (it == "\n") {
            m_messageEdit->insertHtml("<br />");
        } else {
            QString text = it.replace(" ", "&#32;");
            m_messageEdit->insertHtml("<span style=\" font-size:10pt;vertical-align:middle;white-space:pre\">" + it + "</span>");
        }
    }
}

bool MessageFrame::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_messageFrame || object == m_messageLabel) {
        if (event->type() == QEvent::MouseButtonDblClick) {
            if (m_messageContent.sendMessageType == "Image" || m_messageContent.sendMessageType == "File") {
                QDesktopServices::openUrl(QUrl::fromLocalFile(m_messageContent.messageContent));
            }
        }
    }

    if (m_messageEdit != Q_NULLPTR) {
        if (object == m_messageEdit->viewport()) {
            if (event->type() == QEvent::Wheel)
                return true;
        }
    }

    return QFrame::eventFilter(object, event);
}

void MessageFrame::changeEvent(QEvent *event)
{
    if (!m_messageEdit)
        return;

    QTextCursor cursor = m_messageEdit->textCursor();
    cursor.movePosition(QTextCursor::End);
    m_messageEdit->setTextCursor(cursor);

    if ((event->type() == QEvent::PaletteChange) && (this->palette().base().color().rgb() == 4294967295)) {
        if (m_messageContent.sendMessageDirection == "Receive") {
            m_messageEdit->setStyleSheet("background-color:rgba(0,0,0,0.03); border-radius:5px;");
            m_messageEdit->setTextColor(QColor(0, 0, 0, 255));
        }
    }

    if ((event->type() == QEvent::PaletteChange) && (this->palette().base().color().rgb() == 4280821800)) {
        if (m_messageContent.sendMessageDirection == "Receive") {
            m_messageEdit->setStyleSheet("background-color:rgb(100,100,100); border-radius:5px;");
            m_messageEdit->setTextColor(QColor(254, 254, 254).rgb());
        }
    }
}

ChatFrame::ChatFrame(QWidget *parent)
    : DScrollArea(parent)
    , m_historyIndex(0)
    , m_serverName("")
    , m_serverIcon(":/images/Avatar.svg")
    , m_customerIcon(":/images/user.png")
    , m_customerName("me")
{
    QWidget *chatFrameWidget = new QWidget(this);

    m_layout = new QVBoxLayout;
    m_layout->setSpacing(10);
    m_layout->setContentsMargins(20, 0, 20, 20);
    m_layout->addStretch(10);
    chatFrameWidget->setLayout(m_layout);
    this->setWidgetResizable(true);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    m_scrollBar = new QScrollBar;
    this->setVerticalScrollBar(m_scrollBar);
    this->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
    this->setWidget(chatFrameWidget);
}

ChatFrame::~ChatFrame()
{
}

void ChatFrame::setCustomerTitle(const QString &name, const QString &icon)
{
    m_customerIcon = icon;
    m_customerName = name;
}

void ChatFrame::setServiceTitle(const QString &name, const QString &icon)
{
    m_serverName = name;
    if (!icon.isEmpty())
        m_serverIcon = icon;
}

void ChatFrame::setHistoryButton(DCommandLinkButton *historyButton)
{
    m_historyButton = historyButton;
}

void ChatFrame::createMessageLog()
{
    QString oldPath = QDir::currentPath();
    QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/chat/";
    QDir logDir;

    if (!logDir.exists(savePath))
        logDir.mkpath(savePath);

    //将程序的执行路径设置到filePath下
    logDir.setCurrent(savePath);

    //初始化历史消息文件列表
    QFileInfoList fileInfoList = logDir.entryInfoList();
    int totalFileSize = 0;

    foreach (QFileInfo fileInfo, fileInfoList) {
        QString fileExtension = fileInfo.suffix();

        if (fileExtension == "msg") {
            totalFileSize += fileInfo.size();

            if (fileInfo.size() != 0) {
                m_historyList.push_back(fileInfo.completeBaseName());
            }
        }
    }
    if (totalFileSize == 0) {
        m_historyButton->setHidden(true);
    } else {
        m_historyButton->setHidden(false);
    }

    qSort(m_historyList.begin(), m_historyList.end(), sortDate);
    logDir.setPath(oldPath);
}

void ChatFrame::saveMessage(Message message)
{
    QJsonObject rootObject;

    rootObject.insert("datetime", message.sendMessageTime);
    rootObject.insert("type", message.sendMessageType);

    if (message.sendMessageDirection == "Send") {
        rootObject.insert("name", m_customerName);
        rootObject.insert("icon", m_customerIcon);
        message.sendMessageName = m_customerName;
        message.sendMessageIcon = m_customerIcon;
    } else {
        rootObject.insert("name", m_serverName);
        rootObject.insert("icon", m_serverIcon);
        message.sendMessageName = m_serverName;
        message.sendMessageIcon = m_serverIcon;
    }

    rootObject.insert("direct", message.sendMessageDirection);
    rootObject.insert("message_id", message.messageId);
    rootObject.insert("content", message.messageContent);
    rootObject.insert("filename", message.fileName);
    rootObject.insert("filesize", message.fileSize);

    QByteArray data = QJsonDocument(rootObject).toJson().simplified();

    writeLogFile(data);
}

void ChatFrame::writeLogFile(const QByteArray &data)
{
    QString oldPath = QDir::currentPath();
    QString saveName = QDate::currentDate().toString("yyyy.MM.dd") + ".msg";
    QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/chat/";
    QDir logDir;

    if (!logDir.exists(savePath))
        logDir.mkpath(savePath);

    logDir.setCurrent(savePath);

    QFile historyFile;
    historyFile.setFileName(saveName);

    if (!historyFile.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append)) {
        qDebug() << QObject::tr("historyFile open failed");
        return;
    }

    QByteArray secretData = encryptionMessage(data);
    secretData.append("\n");
    historyFile.write(secretData);
    historyFile.close();
    logDir.setPath(oldPath);
}

QByteArray ChatFrame::encryptionMessage(const QByteArray &data)
{
    QByteArray secretData;
    char temp;
    int len = data.length();
    for (int i = 0; i < len; ++i) {
        temp = data.at(i);
        for (int j = 0; j < secretKey.length(); ++j) {
            temp = temp ^ secretKey.at(j);
        }
        secretData.append(temp);
    }

    return secretData;
}

QByteArray ChatFrame::decryptionMessage(const QByteArray &data)
{
    QByteArray decryptData;
    char temp;
    int len = data.length();
    for (int i = 0; i < len; ++i) {
        temp = data.at(i);
        for (int j = (secretKey.length() - 1); j > -1; --j) {
            temp = temp ^ secretKey.at(j);
        }
        decryptData.append(temp);
    }

    return decryptData;
}

//读取历史消息，每次读取一个日期的历史消息
QJsonArray ChatFrame::readMessages(const QString &fileName)
{
    QFile readHistoryFile(fileName);
    QJsonArray histroyArray;

    if (readHistoryFile.open(QFile::ReadOnly)) {
        while (!readHistoryFile.atEnd()) {
            QByteArray secretData = readHistoryFile.readLine();
            QByteArray historyLine = decryptionMessage(secretData);
            historyLine.replace('\0', ""); //替换转义字符
            QJsonParseError jsonError;
            QJsonDocument jsonDocument(QJsonDocument::fromJson(historyLine.simplified(), &jsonError));

            if (jsonError.error != QJsonParseError::NoError) {
                qDebug() << "json error!";
                continue; //读取失败，继续读取下一条记录
            }

            QDateTime lineTime = QDateTime::fromString(jsonDocument.object()["datetime"].toString(), "yyyy.MM.dd hh:mm:ss");
            QDateTime earlyTime = QDateTime::fromString(m_CustomerDateTime, "yyyy.MM.dd hh:mm:ss");

            histroyArray.push_back(jsonDocument.object());
        }
    }

    return histroyArray;
}

void ChatFrame::readHistroy()
{
    QFont fontType;
    fontType.setPointSize(9);
    QString oldPath = QDir::currentPath();
    QString savePath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/chat/";
    QDir::setCurrent(savePath);
    int maxIndex = m_historyList.size() - 1;

    if (m_historyIndex <= maxIndex) {
        QString date = m_historyList.at(m_historyIndex);
        QRegExp pattern("[0-9]+[.]+[0-9]+[.]+[0-9]+$");

        if (!pattern.exactMatch(date))
            return;
        QJsonArray logMessages = readMessages(date + ".msg");

        //如果消息为空，则需要读取下一个日志文件消息．
        if (logMessages.isEmpty())
            return;

        //插入历史消息
        insertHistroyMessages(logMessages);
        //插入历史消息所对应的当日日期
        DLabel *dateLabel = new DLabel;
        dateLabel->setAlignment(Qt::AlignCenter);
        dateLabel->setFont(fontType);
        dateLabel->setText(date);
        m_layout->insertWidget(0, dateLabel);
        m_historyIndex++;

        if (m_historyIndex > maxIndex) {
            m_historyButton->setHidden(true);
        }
    } else {
        m_historyButton->setHidden(true);
    }

    QDir::setCurrent(oldPath);
}

//在当前控件前插入历史消息
void ChatFrame::insertHistroyMessages(QJsonArray messages)
{
    int messageNum = messages.count() - 1;

    for (int i = messageNum; i >= 0; i--) {
        QJsonValue value = messages.takeAt(i);
        QJsonObject object = value.toObject();
        Message message;

        message.sendMessageType = object["type"].toString();
        message.messageId = object["message_id"].toString();
        message.sendMessageDirection = object["direct"].toString();
        message.messageContent = object["content"].toString();
        message.fileName = object["filename"].toString();
        message.fileSize = object["filesize"].toString();
        message.sendMessageName = object["name"].toString();
        message.sendMessageIcon = object["icon"].toString();
        message.sendMessageTime = object["datetime"].toString();

        MessageFrame *messageFrame = new MessageFrame(message, this->widget());
        messageFrame->setMessageRecive();
        m_layout->insertWidget(0, messageFrame);
    }
}

//添加状态消息,显示绘话状态字符串
void ChatFrame::addStatusMessage(const QString &value)
{
    QFont fontType;
    fontType.setPointSize(8);

    QHBoxLayout *timeLayout = new QHBoxLayout;
    QHBoxLayout *contentLayout = new QHBoxLayout;

    DLabel *timeLabel = new DLabel(this);
    DLabel *contentLabel = new DLabel(this);
    timeLabel->setFont(fontType);
    contentLabel->setFont(fontType);
    timeLayout->addWidget(timeLabel);
    contentLayout->addWidget(contentLabel);

    QString messageTime = QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss");
    timeLabel->setText(messageTime);
    QString status = "客服" + m_serverName + "关闭对话";
    if (value == status) {
        QString m_str = tr("Agent ");
        m_str.append(m_serverName);
        m_str.append(tr(" closed the chat"));
        contentLabel->setText(m_str);
    } else if (value == "会话关闭") {
        contentLabel->setText(tr("Session close"));
    } else {
        contentLabel->setText(value);
    }

    QVBoxLayout *messageBoxLayout = new QVBoxLayout;
    messageBoxLayout->addLayout(timeLayout);
    messageBoxLayout->addLayout(contentLayout);
    timeLayout->setAlignment(Qt::AlignHCenter);
    contentLayout->setAlignment(Qt::AlignHCenter);
    timeLayout->setContentsMargins(0, 0, 0, 0);
    contentLayout->setContentsMargins(0, 0, 0, 0);
    messageBoxLayout->setSpacing(0);
    messageBoxLayout->setAlignment(Qt::AlignHCenter);
    messageBoxLayout->setContentsMargins(0, 0, 0, 0);

    int count = m_layout->count();
    m_layout->insertLayout(count - 1, messageBoxLayout);
    m_scrollBar->setRange(0, this->widget()->height());
    m_scrollBar->setValue(m_scrollBar->maximum());
}

//聊天框中添加消息，每次添加在最后面
void ChatFrame::addMessage(Message message)
{
    if (message.sendMessageDirection == "Send") {
        message.sendMessageIcon = m_customerIcon;
        message.sendMessageName = m_customerName;
    } else if (message.sendMessageDirection == "Receive") {
        message.sendMessageIcon = m_serverIcon;
        message.sendMessageName = m_serverName;
    } else {
    }

    message.sendMessageTime = QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss");
    MessageFrame *messageFrame = new MessageFrame(message, this->widget());
    int count = m_layout->count();
    m_layout->insertWidget(count - 1, messageFrame);
    m_scrollBar->setRange(0, this->widget()->height());
    //所有消息列表
    m_messageList.push_back(messageFrame);
    m_scrollBar->setValue(m_scrollBar->maximum());

    //接收到的消息，直接保存。不需要处理传输成功状态
    if (message.sendMessageDirection == "Receive")
        saveMessage(message);
}

void ChatFrame::setCustomerDateTime(const QString &customerTime)
{
    m_CustomerDateTime = customerTime;
}

void ChatFrame::setMessageBack(const QString &messageId)
{
    QList<MessageFrame *>::iterator messagecontent;

    for (messagecontent = m_messageList.begin(); messagecontent != m_messageList.end(); messagecontent++) {
        MessageFrame *curMsgFrame = *messagecontent;

        if (curMsgFrame->getMessageId() == messageId) {
            curMsgFrame->setMessageRecive();
            //发送成功消息保存
            Message message = curMsgFrame->getMessage();
            //发送消息成功，保存当前消息
            saveMessage(message);
            break;
        }
    }
}

CustomerChatWidget::CustomerChatWidget(QWidget *parent)
    : DFrame(parent)
    , m_sessionId(0)
    , m_customerToken("")
    , m_userUuid("")
    , m_customSessionCode(0)
{
    initUI();
    initEmtionMap();
    getSystemIconName();
    this->installEventFilter(this);
    m_timer = new QTimer(this);
    m_timer->setSingleShot(true);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(delayTriggel()));
}

CustomerChatWidget::~CustomerChatWidget()
{
    closeSession();
}

void CustomerChatWidget::getSystemIconName()
{
    QDBusInterface manual("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts/User1000", "com.deepin.daemon.Accounts.User", QDBusConnection::systemBus());

    m_icon = manual.property("IconFile").toString();

    if (m_icon.startsWith("file"))
        m_icon = m_icon.mid(7);

    m_name = manual.property("UserName").toString();
    m_userUuid = manual.property("UUID").toString();
    m_chatWidget->setCustomerTitle(m_name, m_icon);
}

//布局界面
void CustomerChatWidget::initUI()
{
    QFont fontType;
    fontType.setPointSize(10);

    this->setLineWidth(0);
    DFrame *horizontalLineOne = new DFrame;
    horizontalLineOne->setFixedSize(620, 2);
    horizontalLineOne->setBackgroundRole(DPalette::ItemBackground);

    DFrame *horizontalLineTwo = new DFrame;
    horizontalLineTwo->setFixedSize(620, 2);
    horizontalLineTwo->setBackgroundRole(DPalette::ItemBackground);

    m_titleWidget = new TitleFrame(this);
    m_titleWidget->setFrameRounded(false);
    m_titleWidget->setFrameShape(QFrame::NoFrame);

    QFont historyType;
    historyType.setPointSize(8);
    m_historyButton = new DCommandLinkButton(tr("View history"), this);
    m_historyButton->setFont(historyType);
    m_historyButton->setFocusPolicy(Qt::NoFocus);

    QHBoxLayout *historyLayout = new QHBoxLayout;
    historyLayout->addWidget(m_historyButton);
    historyLayout->setAlignment(Qt::AlignHCenter);

    m_chatWidget = new ChatFrame(this);
    m_chatWidget->setFixedHeight(300);
    m_chatWidget->setFrameShape(QFrame::NoFrame);
    //首先设置历史按键
    m_chatWidget->setHistoryButton(m_historyButton);
    //其次读取创建历史消息文件
    m_chatWidget->createMessageLog();
    connect(m_historyButton, &DCommandLinkButton::clicked, m_chatWidget, &ChatFrame::readHistroy);

    DFrame *buttonFrame = new DFrame(this);
    buttonFrame->setFrameRounded(false);
    buttonFrame->setFrameShape(QFrame::NoFrame);
    m_openFileButton = new DIconButton(this);
    m_openFileButton->setIcon(QIcon::fromTheme("uss_file"));
    m_openFileButton->setFlat(true);
    m_openFileButton->setFixedSize(17, 17);
    m_openFileButton->setIconSize(QSize(16, 16));
    m_openFileButton->setFocusPolicy(Qt::NoFocus);
    connect(m_openFileButton, &DIconButton::clicked, this, &CustomerChatWidget::openFile);

    QHBoxLayout *phlayout = new QHBoxLayout;
    phlayout->addWidget(m_openFileButton);
    phlayout->addStretch();
    buttonFrame->setLayout(phlayout);
    phlayout->setContentsMargins(10, 0, 10, 0);
    buttonFrame->setContentsMargins(0, 0, 0, 0);

    m_inputFrame = new DFrame(this);
    m_inputFrame->setLineWidth(0);
    m_userInputWidget = new QTextEdit(this);
    m_userInputWidget->setFrameShape(QFrame::NoFrame);
    onFontSizecurrentIndexChanged("9");            //设置插入HTML格式
    connect(m_userInputWidget, &QTextEdit::textChanged, this, [ = ](){
        onFontSizecurrentIndexChanged("9");
    });

    m_userInputWidget->setPlaceholderText(tr("Welcome to contact us,please describe your questions,press Enter to send messages"));
    m_userInputWidget->setFont(fontType);
    m_userInputWidget->setFixedHeight(100);

    QHBoxLayout *inputWidgetLayout = new QHBoxLayout;
    inputWidgetLayout->setContentsMargins(0, 0, 0, 0);
    inputWidgetLayout->addSpacing(20);
    inputWidgetLayout->addWidget(m_userInputWidget);
    inputWidgetLayout->addSpacing(10);
    m_inputFrame->setLayout(inputWidgetLayout);

    m_enterTipButton = new DPushButton(tr("Send"));
    m_enterTipButton->setFont(fontType);
    m_enterTipButton->setFixedSize(90, 36);
    m_enterTipButton->setFocusPolicy(Qt::NoFocus);
    connect(m_enterTipButton, &DPushButton::clicked, this, [ = ]() {
        if (LicenseStatus::authorizeStatus) {
            sendMessages();
        } else {
            notifyActivator();
        }
    });

    QHBoxLayout *enterTipLayout = new QHBoxLayout;
    inputWidgetLayout->setContentsMargins(0, 0, 0, 0);
    enterTipLayout->addStretch();
    enterTipLayout->addWidget(m_enterTipButton);
    enterTipLayout->addSpacing(10);

    QVBoxLayout *overallLayout = new QVBoxLayout;
    overallLayout->setContentsMargins(0, 0, 0, 0);
    overallLayout->addWidget(m_titleWidget);
    overallLayout->addWidget(horizontalLineOne);
    overallLayout->addStretch();
    overallLayout->addLayout(historyLayout);
    overallLayout->addWidget(m_chatWidget);
    overallLayout->addWidget(horizontalLineTwo);
    overallLayout->addWidget(buttonFrame);
    overallLayout->addWidget(m_inputFrame);
    overallLayout->addLayout(enterTipLayout);
    overallLayout->addSpacing(10);
    setLayout(overallLayout);

    m_inputFrame->setAcceptDrops(true);
    m_inputFrame->setMouseTracking(true);
    m_inputFrame->installEventFilter(this);
    m_userInputWidget->setAcceptDrops(false);
    m_userInputWidget->installEventFilter(this);
}

void CustomerChatWidget::initEmtionMap()
{
    int i = 0;
    QString value;
    QFile file(":/images/emotion/emotion_escape.txt");
    if (!file.open(QIODevice::ReadOnly))
        return;

    while (!file.atEnd()) {
        value = file.readLine().trimmed();
        nameToEscape.insert(QString::number(i) + ".png", value);
        escapeToName.insert(value, QString::number(i) + ".png");
        i++;
    }
}

bool CustomerChatWidget::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_userInputWidget) {
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = dynamic_cast<QKeyEvent *>(event);
            if (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) {
                if (LicenseStatus::authorizeStatus) {
                    sendMessages();
                } else {
                    notifyActivator();
                }
                return true;
            }
        }
    } else if (object == this) {
        if (event->type() == QEvent::Show) {
            m_timer->start(1);
        }
    } else if (object == m_inputFrame) {
        if (event->type() == QEvent::DragEnter) {
            QDragEnterEvent *enterEvent = dynamic_cast<QDragEnterEvent *>(event);

            if (enterEvent->mimeData()->hasFormat("text/uri-list")) {
                QList<QUrl> urls = enterEvent->mimeData()->urls();

                foreach (QUrl url, urls) {
                    QFileInfo fi(url.toLocalFile());
                    //屏蔽文件夹和svg文件，不支持拖拽文件夹和svg文件
                    if (fi.isDir() || (fi.fileName().split(".").last() == "svg")) {
                        enterEvent->ignore();
                        return true;
                    }
                }
                enterEvent->acceptProposedAction();
            } else {
                enterEvent->ignore();
            }
        }
        if (event->type() == QEvent::Drop) {
            QDropEvent *dropEvent = dynamic_cast<QDropEvent *>(event);
            //获取MIME数据
            const QMimeData *mineData = dropEvent->mimeData();
            //获取URL列表
            QList<QUrl> urlList = mineData->urls();

            if (urlList.isEmpty())
                return true;
            //将其中第一个URL表示为本地文件路径
            QString fileName = urlList.at(0).toLocalFile();
            if (fileName.isEmpty())
                return true;

            if (!fileName.isEmpty()) {
                if (LicenseStatus::authorizeStatus) {
                    sendFile(fileName);
                } else {
                    notifyActivator();
                }
            }
            return true;
        }
    } else {
    }

    return QFrame::eventFilter(object, event);
}

void CustomerChatWidget::sendFile(const QString &url)
{
    QFont fontType;
    fontType.setPointSize(9);
    QFileInfo fileInfo(url);

    if (fileInfo.size() > FILESIZE) {
        DDialog promptDialog(this);
        promptDialog.addButton(tr("OK"));
        promptDialog.setIcon(QIcon::fromTheme("uss_warning"));
        promptDialog.setMessage(tr("No more than 10 MB please"));
        promptDialog.exec();
        return;
    }

    if (fileInfo.size() == 0) {
        DDialog promptDialog(this);
        promptDialog.addButton(tr("OK"));
        promptDialog.setIcon(QIcon::fromTheme("uss_warning"));
        promptDialog.setMessage(tr("Attachments cannot be files with 0 bytes"));
        promptDialog.exec();
        return;
    }

    if(fileInfo.fileName().split(".").last() == "svg") {
        DDialog promptDialog(this);
        promptDialog.addButton(tr("OK"));
        promptDialog.setIcon(QIcon::fromTheme("uss_warning"));
        promptDialog.setMessage(tr(".svg files are not supported"));
        promptDialog.exec();
        return;
    }

    Message message;
    message.messageId = QUuid::createUuid().toString();
    message.sendMessageDirection = "Send";
    message.messageContent = fileInfo.filePath();
    QString type;
    QString fileExtension = fileInfo.completeSuffix();

    if (fileExtension == "png" || fileExtension == "jpg" || fileExtension == "jpeg") {
        message.sendMessageType = "Image";
    } else {
        message.sendMessageType = "File";
    }

    message.fileName = fileInfo.fileName();
    message.fileSize = QString("%1").arg(fileInfo.size());

    if (!DNetState::getInstance()->getNetState()) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("network unavailable,message can not send Successful"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);

        return;
    }

    m_chatWidget->addMessage(message);

    if (m_customSessionCode == BUSYCODE) {
        DFloatingMessage *floatMessage = new DFloatingMessage();
        floatMessage->setFont(fontType);
        floatMessage->setDuration(800);
        floatMessage->setMessage(tr("Customer service is busy, please wait patiently"));

        DMessageManager::instance()->sendMessage(m_chatWidget, floatMessage);

        m_messageUnArrived.push_back(message);
    } else if (m_customSessionCode == NOONLINECODE) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("Online service is not available now"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
    } else {
        if (m_sessionId == 0) {
            createSession();
            m_messageUnArrived.push_back(message);
        } else {
            emit uploadFile(url, message.messageId);
        }
    }
}

void CustomerChatWidget::sendMessages()
{
    QFont fontType;
    fontType.setPointSize(9);
    QStringList messageList = getInputEditData();
    QString messageContent = escapeCharacterSubstitution(messageList);

    //把最后一个回车换行符删掉
    while (messageContent.endsWith("\n")) {
        messageContent.remove(messageContent.length() - 2, 2);
    }

    if (messageContent.trimmed().isEmpty()) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(500);
        floatingMessage->setMessage(tr("Cannot send a blank message"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
        return;
    }

    if (messageContent.length() > SUCCESSCODE) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("Cannot send message count > 1000"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
        return;
    }

    Message message;
    QJsonObject rootObject;
    QString messageId = QUuid::createUuid().toString().remove("{").remove("}");
    rootObject.insert("customer_token", m_customerToken);
    rootObject.insert("im_sub_session_id", m_sessionId);
    rootObject.insert("message_id", messageId);
    rootObject.insert("type", "message");

    message.sendMessageDirection = "Send";
    message.sendMessageType = "Message";
    message.messageId = messageId;
    message.messageContent = messageContent.trimmed();

    QJsonObject dataObject;
    dataObject.insert("content", messageContent.trimmed());
    rootObject.insert("data", QJsonValue(dataObject));
    m_userInputWidget->clear();
    m_userInputWidget->selectAll();
    m_userInputWidget->setCurrentFont(fontType);
    if (!DNetState::getInstance()->getNetState()) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("network unavailable, message can not send Successful"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
        return;
    }

    if (m_customSessionCode == BUSYCODE) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("Customer service is busy, please wait patiently"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
        m_messageUnArrived.push_back(message);

    } else if (m_customSessionCode == NOONLINECODE) {
        DFloatingMessage *floatingMessage = new DFloatingMessage();
        floatingMessage->setFont(fontType);
        floatingMessage->setDuration(800);
        floatingMessage->setMessage(tr("Online service is not available now"));
        DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);

    } else {
        if (m_sessionId == 0) {
            createSession();
            m_messageUnArrived.push_back(message);
            return;
        } else {
            emit sendMessageSignals(QJsonDocument(rootObject).toJson().simplified()); //发送消息
        }
    }
    m_chatWidget->addMessage(message);
}

void CustomerChatWidget::openFile()
{
    QString url = QFileDialog::getOpenFileName(this, tr("Open File"), QStandardPaths::writableLocation(QStandardPaths::HomeLocation));

    if (!url.isEmpty()) {
        if (LicenseStatus::authorizeStatus) {
            sendFile(url);
        } else {
            notifyActivator();
        }
    }
}

void CustomerChatWidget::uploadFileSuccess(const QString &webUrl, const QByteArray &data)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if (json_error.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject jsonObject = jsonDoc.object();
    QString uploadPath = jsonObject["fileurl"].toString();
    QString message_id = jsonObject["message_id"].toString();
    uploadPath = webUrl + "/" + uploadPath;
    QFileInfo info(uploadPath);
    double fileSize = jsonObject["filesize"].toDouble();
    QString fileName = jsonObject["filename"].toString();
    QString type;

    if (fileName.contains("jpg") || fileName.contains("jpeg") || fileName.contains("png")) {
        type = "image";
    } else {
        type = "file";
    }

    //组织文件发送 json结构
    QJsonObject rootObject;
    rootObject.insert("customer_token", m_customerToken);
    rootObject.insert("im_sub_session_id", m_sessionId);
    rootObject.insert("message_id", message_id);
    rootObject.insert("type", type);

    QJsonObject dataObject;
    dataObject.insert("content", uploadPath);
    dataObject.insert("filename", fileName);

    QString size;
    QString unit = "B";
    if (fileSize > 1024) {
        fileSize /= 1024;
        unit = "Kb";
        if (fileSize > 1024) {
            fileSize /= 1024;
            unit = "Mb";
            if (fileSize > 1024) {
                fileSize /= 1024;
                unit = "Gb";
            }
        }
    }

    size.append(QString::number(fileSize)).append(unit);
    dataObject.insert("filesize", size);
    rootObject.insert("data", QJsonValue(dataObject));

    emit sendMessageSignals(QJsonDocument(rootObject).toJson().simplified()); //发送文件消息
}

QString CustomerChatWidget::getCustomSesson()
{
    QString uid = m_userUuid.left(8);
    return "uos-" + m_name + "-" + uid;
}

void CustomerChatWidget::createSession()
{
    //customertoken 每次会话创建
    m_customerToken = getCustomSesson();

    QJsonObject jsonObject;
    jsonObject.insert("customer_token", m_customerToken);

    emit sendStartSession(QJsonDocument(jsonObject).toJson().simplified());
    m_chatWidget->setCustomerDateTime(QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss"));
}

//客户主动关闭客服
void CustomerChatWidget::closeSession()
{
    Message message;
    QJsonObject rootObject;

    QString messageId = QUuid::createUuid().toString().remove("{").remove("}");
    rootObject.insert("customer_token", m_customerToken);
    rootObject.insert("im_sub_session_id", m_sessionId);
    rootObject.insert("message_id", messageId);
    rootObject.insert("type", "close");

    message.sendMessageDirection = "Send";
    message.sendMessageType = "close";
    message.messageId = messageId;
    message.messageContent = tr("Session close");

    QJsonObject dataObject;
    dataObject.insert("content", message.messageContent);
    rootObject.insert("data", QJsonValue(dataObject));

    emit sendMessageSignals(QJsonDocument(rootObject).toJson().simplified());
}

QString CustomerChatWidget::downServiceFile(const QString &urlPath)
{
    QString oldPath = QDir::currentPath();
    QFileInfo fileInfo(urlPath);
    QString saveName = fileInfo.fileName();
    QString saveDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
    QDir CacheDir;

    if (!CacheDir.exists(saveDir))
        CacheDir.mkpath(saveDir);

    QDir::setCurrent(saveDir);
    QString savepath = saveDir + "/" + saveName;
    //构造请求
    QNetworkAccessManager manager;
    QNetworkRequest request;
    request.setUrl(urlPath);
    //发送请求
    QNetworkReply *reply = manager.get(request);
    //开启一个局部的事件循环，等待响应结束，退出
    QEventLoop loop;
    connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    //判断是否出错,出错则结束
    if (reply->error() != QNetworkReply::NoError) {
        qDebug() << "QNetworkReply Error";
        return QString("");
    }

    QFile file(saveName);
    if (!file.open(QIODevice::WriteOnly)) {
        qDebug() << "ServiceFile open fail";
        return ("");
    }

    file.write(reply->readAll());
    file.close();
    delete reply;
    QDir::setCurrent(oldPath);

    return savepath;
}

//会话失败 2001 2002
void CustomerChatWidget::sessionFault(const int &code, const QString &errorStr)
{
    m_customSessionCode = code;
    if (code == BUSYCODE) {
        m_chatWidget->addStatusMessage(tr("Customer service is busy, please wait patiently"));
        emit createSocket(m_customerToken);
    }

    if (code == NOONLINECODE) {
        m_chatWidget->addStatusMessage(tr("Online service is not available now"));
    }
}

//会话创建完成数据传递  1000
void CustomerChatWidget::sessionStart(const QByteArray &data)
{
    m_customSessionCode = SUCCESSCODE;
    QJsonParseError jsonError;
    QJsonDocument jsonDocument(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject rootObject = jsonDocument.object();
    QString sessionMessage = rootObject["message"].toString();
    QJsonObject jsonObject = rootObject["assign_info"].toObject();
    QString agentName = jsonObject["agent_name"].toString();
    QString iconPath = jsonObject["agent_avatar"].toString();
    m_sessionId = jsonObject["im_sub_session_id"].toDouble();
    QString downPath = downServiceFile(iconPath);
    m_chatWidget->setServiceTitle(agentName, downPath);
    m_titleWidget->setTitle(agentName, downPath);

    //会话创建成功之后，发送消息传递custom_token
    emit createSocket(m_customerToken);
    m_chatWidget->addStatusMessage(tr("Session created successfully"));
}

void CustomerChatWidget::postUnArrivedMessage()
{
    if (m_sessionId == 0)
        return;

    if (m_messageUnArrived.count() > 0) {
        for (int i = 0; i < m_messageUnArrived.count(); i++) {
            Message message = m_messageUnArrived.at(i);

            if (!DNetState::getInstance()->getNetState()) {
                QFont fontType;
                fontType.setPointSize(9);
                DFloatingMessage *floatingMessage = new DFloatingMessage();
                floatingMessage->setFont(fontType);
                floatingMessage->setDuration(800);
                floatingMessage->setMessage(tr("network unavailable, message can not send Successful"));
                DMessageManager::instance()->sendMessage(m_chatWidget, floatingMessage);
                return;
            }

            QJsonObject rootObject;
            rootObject.insert("customer_token", m_customerToken);
            rootObject.insert("im_sub_session_id", m_sessionId);
            rootObject.insert("message_id", message.messageId);
            rootObject.insert("type", message.sendMessageType.toLower());

            QJsonObject dataObject;
            dataObject.insert("content", message.messageContent);
            rootObject.insert("data", QJsonValue(dataObject));

            emit sendMessageSignals(QJsonDocument(rootObject).toJson().simplified()); //发送文件消息
        }
        m_messageUnArrived.clear();
    }
}

QList<QStringList> CustomerChatWidget::parseDocNode(QDomNode node)
{
    QFont font;
    font.setPixelSize(14);
    QFontMetrics fontMetrics(font);
    QList<QStringList> attribute;
    const QDomNodeList list = node.childNodes();
    for (int i = 0; i < list.count(); i++) {
        const QDomNode node = list.at(i);
        if (node.isElement()) {
            const QDomElement element = node.toElement();
            if (node.hasChildNodes()) {
                attribute << parseDocNode(node);
            }
            if (element.tagName() == "br") {
                QStringList attributeLst;
                attributeLst << "br" << "\n";
                attribute << attributeLst;
            }
            if (element.tagName() == "img") {
                QStringList attributeLst;
                attributeLst << "img" << element.attribute("src");
                attribute << attributeLst;
            }
            if (element.tagName() == "span") {
                QStringList attributeLst;
                attributeLst << "text" << element.text();
                attribute << attributeLst;
            }
        }
    }
    return attribute;
}

void CustomerChatWidget::onFontSizecurrentIndexChanged(QString size)
{
    QTextCharFormat fmt;
    fmt.setFontPointSize(size.toInt());
    QTextCursor cursor = m_userInputWidget->textCursor();
    cursor.mergeCharFormat(fmt);
    m_userInputWidget->setFontPointSize(size.toInt());
    m_userInputWidget->setFocus();
}

QString CustomerChatWidget::escapeCharacterSubstitution(QStringList message)
{
    QString sendMessage;
    for (auto it : message) {
        QString imgName = it.mid(it.lastIndexOf('/') + 1);
        if (!nameToEscape.value(imgName).isEmpty()) {
            sendMessage += nameToEscape.value(imgName);
            continue;
        }
        sendMessage += it;
    }
    return  sendMessage;
}

QStringList CustomerChatWidget::getInputEditData()
{
    QStringList messageList;
    const QString&& html = m_userInputWidget ->document()->toHtml();

    QDomDocument doc;
    doc.setContent(html);
    QDomElement root = doc.documentElement();
    QDomNode node = root.firstChildElement("body");
    QList<QStringList> msgLst = parseDocNode(node);

    QString qsMsg;
    for (int index = 0; index < msgLst.size(); index++) {
        if (msgLst.at(index).at(0) == "img") {
            QString imagePath = msgLst.at(index).at(1);
            qsMsg += imagePath;
            messageList.append(imagePath);
        } else if (msgLst.at(index).at(0) == "br") {
            qsMsg += msgLst.at(index).at(1);
            messageList.append(msgLst.at(index).at(1));
        }else if (msgLst.at(index).at(0) == "text") {
            qsMsg += msgLst.at(index).at(1);
            messageList.append(msgLst.at(index).at(1));
        }
    }

    return messageList;
}

void CustomerChatWidget::notifyActivator()
{
    QStringList actions = QStringList() << "_open" << tr("Activate");
    QVariantMap hints;
    hints["x-deepin-action-_open"] =
        "dbus-send,--print-reply,--dest=com.deepin.license.activator,/com/deepin/license/activator,com.deepin.license.activator.Show";

    QList<QVariant> argumentList;
    argumentList << "uos-activator";
    argumentList << static_cast<uint>(0);
    argumentList << "uos-activator-false";
    argumentList << "";
    argumentList << tr("Your system is not activated,To enjoy this service, please activate first");
    argumentList << actions;
    argumentList << hints;
    argumentList << static_cast<int>(3000);

    static QDBusInterface notifyApp("org.freedesktop.Notifications",
                                    "/org/freedesktop/Notifications",
                                    "org.freedesktop.Notifications");

    notifyApp.asyncCallWithArgumentList("Notify", argumentList);
}

void CustomerChatWidget::delayTriggel()
{
    if (!m_sessionId)
        createSession();
}

void CustomerChatWidget::textChangedSlot(QString it)
{
    qDebug() << "this is textChangedSlot ";
    m_userInputWidget->insertHtml("<span style=\" font-size:10pt;vertical-align:middle;\">" + it + "</span>");
}

QStringList MessageFrame::parseEmtion(const QString src)
{
    int pos;
    //表情所在的位置和其转义字符长度
    QMap<int, int> emtionPos;
    for (auto it = nameToEscape.begin(); it != nameToEscape.end(); it++) {
        if (src.contains(it.value())) {
            pos = 0;
            while (pos < src.length()) {
                pos = src.indexOf(it.value(), pos);
                if (pos != -1) {
                    if (emtionPos.value(pos) < it.value().length())
                        emtionPos.insert(pos, it.value().length());
                    pos = pos + it.value().length();
                } else {
                    break;
                }
            }
        }

        if (src.contains("\n")) {
            pos = 0;
            while (pos < src.length()) {
                pos = src.indexOf("\n", pos);
                if (pos != -1) {
                    emtionPos.insert(pos, 1);
                    pos = pos + 1;
                } else {
                    break;
                }
            }
        }
    }

    int startPos = 0;
    QStringList parseDoneMessage;
    qDebug() << "emtionPos.............." << emtionPos;

    if (!emtionPos.isEmpty()) {
        for (auto it = emtionPos.begin(); it != emtionPos.end(); it++) {
            if ((it.key() - startPos) != 0)
                parseDoneMessage.push_back(src.mid(startPos, (it.key() - startPos)));

            parseDoneMessage.push_back(src.mid(it.key(), it.value()));
            startPos = it.key() + it.value();
        }

        int lastEmtionPos = emtionPos.lastKey() + emtionPos.value(emtionPos.lastKey());
        if ((src.length() - lastEmtionPos) != 0)
            parseDoneMessage.push_back(src.mid(lastEmtionPos));
        parseDoneMessage = escapeTransformPath(parseDoneMessage);
    } else {
        parseDoneMessage.push_back(src);
    }

    return parseDoneMessage;
}

QStringList MessageFrame::escapeTransformPath(QStringList &emotionList)
{
    for (auto it = nameToEscape.begin(); it != nameToEscape.end(); it++) {
        for (int i = 0; i < emotionList.length(); i++) {
            if (emotionList.at(i) == it.value()) {
                emotionList.replace(i, ":/images/emotion/" + it.key());
            }
        }
    }

    return emotionList;
}

//用户消息回传
void CustomerChatWidget::recieveMessageRet(const QByteArray &data)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDocument(QJsonDocument::fromJson(data.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject jsonObject = jsonDocument.object();
    QString messageId = jsonObject["message_id"].toString();
    m_chatWidget->setMessageBack(messageId);
}

void CustomerChatWidget::createSocketSlot()
{
    postUnArrivedMessage();
}

//服务端消息回传回来时，出发添加服务消息槽函数
void CustomerChatWidget::recieveMessage(const QByteArray &messageRecieve)
{
    QJsonParseError jsonError;
    QJsonDocument jsonDocument(QJsonDocument::fromJson(messageRecieve.simplified(), &jsonError));

    if (jsonError.error != QJsonParseError::NoError) {
        qDebug() << "json error!";
        return;
    }

    QJsonObject jsonObject = jsonDocument.object();
    QJsonArray array = jsonObject["messages"].toArray();
    double oldSessionId = m_sessionId;
    double sessionId = 0;
    for (int i = 0; i < array.size(); i++) {
        QJsonValue arrayValue = array.at(i);
        if (arrayValue.isObject()) {
            QJsonObject object = arrayValue.toObject();
            QString type = object["type"].toString();
            QString messageId = object["message_id"].toString();
            QJsonObject messageData = object["data"].toObject();
            QString name = object["agent_name"].toString();
            QString icon = object["agent_avatar"].toString();
            sessionId = object["im_sub_session_id"].toDouble();

            Message message;
            message.sendMessageName = name;
            QString downPath = downServiceFile(icon);
            message.sendMessageIcon = downPath;
            m_chatWidget->setServiceTitle(message.sendMessageName, downPath);
            m_titleWidget->setTitle(message.sendMessageName, downPath);
            message.messageId = messageId;
            message.sendMessageDirection = "Receive";

            if (type == "image") {
                m_customSessionCode = SUCCESSCODE;
                QString content = messageData["content"].toString();
                message.sendMessageType = "Image";
                QString filePath = downServiceFile(content);
                message.messageContent = filePath;
                m_chatWidget->addMessage(message);
                m_sessionId = sessionId;
            } else if (type == "file") {
                m_customSessionCode = SUCCESSCODE;
                message.sendMessageType = "File";
                QString content = messageData["content"].toString();
                QString fileSize = messageData["filesize"].toString();
                QString fileName = messageData["filename"].toString();
                QString filePath = downServiceFile(content);
                message.messageContent = filePath;
                message.fileName = fileName;
                message.fileSize = fileSize;
                m_chatWidget->addMessage(message);
                m_sessionId = sessionId;
                //发送消息
            } else if (type == "message") {
                m_customSessionCode = SUCCESSCODE;
                message.sendMessageType = "Message";
                message.messageContent = messageData["content"].toString();
                m_chatWidget->addMessage(message);
                m_sessionId = sessionId;
                //客服关闭会话
            } else if (type == "close") {
                message.messageContent = messageData["content"].toString();
                m_sessionId = 0;
                m_customSessionCode = 0;

                emit closeWebSocket();
                m_chatWidget->addStatusMessage(message.messageContent);
            } else {
            }
        }
    }
    //客户信息从客服排队中被接入处理
    if ((oldSessionId == 0) && (m_customSessionCode == SUCCESSCODE) && sessionId != 0) {
        m_sessionId = sessionId;
        postUnArrivedMessage();
    }
}
