/*
 * Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
 *
 * Author:     niuningjia <niuningjia@uniontech.com>
 *
 * Maintainer: niuningjia <niuningjia@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CUSTOMERCHATWIDGET_H
#define CUSTOMERCHATWIDGET_H

#include <DWidget>
#include <DFrame>
#include <DScrollArea>
#include <DTextEdit>
#include <DIconButton>
#include <DLabel>
#include <DCommandLinkButton>

#include <QList>
#include <QTableWidget>
#include <QDomDocument>

DWIDGET_USE_NAMESPACE

class WebSocketClient;
class QVBoxLayout;
class QHBoxLayout;

typedef struct Message {
    QString sendMessageName;
    QString sendMessageIcon;
    QString sendMessageTime;
    QString sendMessageType;
    QString sendMessageDirection;
    QString messageId;
    QString messageContent;
    QString fileName;
    QString fileSize;
} Message;

//表情图片名对应的转义字符
static QMap<QString,QString> nameToEscape;
//转义字符对应图片名
static QMap<QString,QString> escapeToName;

class TitleFrame : public DFrame
{
    Q_OBJECT
public:
    explicit TitleFrame(QWidget *parent = Q_NULLPTR);
    ~TitleFrame();

    void setTitle(const QString &customerName, const QString &customerIcon);

private:
    QString m_customerName;
    QString m_customerIcon;
    DIconButton *m_customerAvatar;
    DLabel *m_customerAppellation;
};

class MessageFrame : public DFrame
{
    Q_OBJECT
public:
    explicit MessageFrame(Message message, QWidget *parent = Q_NULLPTR);
    ~MessageFrame();

    void setMessageRecive();
    QString getMessageId();
    Message getMessage();
    DFrame *createFileFrame();
    DLabel *createImageLabel();

protected:
    bool eventFilter(QObject *object, QEvent *event);
    void changeEvent(QEvent *event);

private slots:
    void saveAsFile();
    void openFile();
    void adjustContent();
    QStringList adjustRect(QStringList messageList);
    void adjustText();
    void inputContent(QStringList messageList);
    //将表情字符串解析为字符链表
    QStringList parseEmtion(const QString src);
    //将表情字符替换为图片路径
    QStringList escapeTransformPath(QStringList &emotionList);
    //解析换行符

private:
    void initUI();

    DMenu *m_menu;
    Message m_messageContent;
    QHBoxLayout *m_messageFrameLayout;
    DIconButton *m_messageSuccessLabel;
    DLabel *m_messageLabel;
    DFrame *m_messageFrame;
    bool m_sendMessageSuccess;
    DLabel *m_timeLabel;
    DIconButton *m_iconLabel;
    DLabel *m_nameLabel;
    QTextEdit *m_messageEdit;
};

class ChatFrame : public DScrollArea
{
    Q_OBJECT

public:
    explicit ChatFrame(QWidget *parent = Q_NULLPTR);
    ~ChatFrame();

    void setCustomerTitle(const QString &name, const QString &icon);
    void setServiceTitle(const QString &name, const QString &icon);
    void setHistoryButton(DCommandLinkButton *historyButton);
    //创建日志文件
    void createMessageLog();
    //添加消息
    void addMessage(Message message);
    //添加状态消息
    void addStatusMessage(const QString &value);
    void insertHistroyMessages(QJsonArray messages);
    QJsonArray readMessages(const QString &fileName);
    //设置消息回传成功标志
    void setMessageBack(const QString &messageId);
    void setCustomerDateTime(const QString &customerTime);

public slots:
    void readHistroy();

private:
    void saveMessage(Message message);
    void writeLogFile(const QByteArray &data);
    //对聊天历史记录加密
    QByteArray encryptionMessage(const QByteArray &data);
    //对聊天历史记录解密
    QByteArray decryptionMessage(const QByteArray &data);

    QVBoxLayout *m_layout;
    QScrollBar *m_scrollBar;
    DLabel *m_historyLabel;
    QList<MessageFrame *> m_messageList;
    QList<QString> m_historyList;
    int m_historyIndex;
    QString m_serverName;
    QString m_serverIcon;
    QString m_customerIcon;
    QString m_customerName;
    QString m_CustomerDateTime;
    DCommandLinkButton *m_historyButton;
};

class CustomerChatWidget : public DFrame
{
    Q_OBJECT
public:
    explicit CustomerChatWidget(QWidget *parent = Q_NULLPTR);
    ~CustomerChatWidget();

public slots:
    void createSocketSlot();
    //接受到客服消息
    void recieveMessage(const QByteArray &data);
    //选择文件，发送文件
    void openFile();
    //会话失败
    void sessionFault(const int &code, const QString &errorStr);
    //会话成功返回
    void sessionStart(const QByteArray &data);
    //用户消息回传
    void recieveMessageRet(const QByteArray &data);
    void uploadFileSuccess(const QString &webUrl, const QByteArray &data);
    void delayTriggel();
    void textChangedSlot(QString it);

protected:
    bool eventFilter(QObject *object, QEvent *event);

private:
    void initUI();
    void initEmtionMap();
    void getSystemIconName();
    QString getCustomSesson();
    QString downServiceFile(const QString &urlPath);
    void sendMessages();
    void sendFile(const QString &url);
    void createSession();
    void closeSession();
    void postUnArrivedMessage();
    QList<QStringList> parseDocNode(QDomNode node);
    void onFontSizecurrentIndexChanged(QString size);
    //路径转换转义字符
    QString escapeCharacterSubstitution(QStringList message);
    //获取输入框中的数据
    QStringList getInputEditData();
    void notifyActivator();

    TitleFrame *m_titleWidget;
    ChatFrame *m_chatWidget;
    DFrame *m_inputFrame;
    QTextEdit *m_userInputWidget;
    DPushButton *m_enterTipButton;
    DIconButton *m_openFileButton;
    double m_sessionId;
    QString m_customerToken;
    DCommandLinkButton *m_historyButton;
    QString m_icon;
    QString m_name;
    QString m_userUuid;
    int m_customSessionCode;
    QList<Message> m_messageUnArrived;
    QTimer *m_timer;

signals:
    void uploadFile(const QString &url, const QString &messageId);
    void sendMessageSignals(const QByteArray &data);
    //发送满意度
    void sendSurvey(const QByteArray &data);
    void sendStartSession(const QByteArray &data);
    void createSocket(const QString &customerToken);
    void closeWebSocket();
};

#endif // CUSTOMERCHATWIDGET_H
